/***************************************************************************
* A(lchemist) Game, a game about taking elements from the surrounding and throwing to a your enemies.
* Copyright (C) 2012  Sven Stamm
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
***************************************************************************/
package arakash.lpc.Map;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Random;

import org.newdawn.slick.SlickException;
import org.newdawn.slick.geom.Vector2f;
import org.newdawn.slick.tiled.TiledMap;
import org.newdawn.slick.util.pathfinding.PathFindingContext;
import org.newdawn.slick.util.pathfinding.TileBasedMap;

/**
 * the GameMap class loads tiled Maps created with the free 'Tiled' Programm or any other program who can create tiledmaps(.tmx)
 * You can and probably have to set some Options for the map to work correctly
 * current supported Map properties: <br>
 * <b>layerbelowplayer</b>: used to indicate witch layer is to be drawn beneath the player and witch above(like treetops)
 * <br>
 * Your Map will need one 'collision' Layer to detect collisions. currently supported Layer Propertys:<br>
 * <b>collision</b>: "1", if this layer is to be used as collision layer, meaning entities can't enter a tile present at this layer.
 * additionally you will have to place 'SpawnPoint' Objects for the Player to spawn in and 'EnemySpawn' Objects for enemys to spawn. 
 * current supported Object Types:
 * <b>SpawnPoint</b>: The Player will spawn in a random location of a random SpawnPoint <br>
 * <b>EnemySpawn</b> : Monster will spawn in a random location of a random EnemySpawn <br>
 * <b>NPCSpawn</b> : the server will put NPCs here, like Questgivers.<br>
 * <br>
 * Please note that you should make sure, that your spawnpoints are not able to spawn entites on a collision Tile to prevent them from getting stuck.
 * @author Sven Stamm
 */
public class GameMap extends TiledMap implements TileBasedMap {
	
	private int mapTotalWidth;
	private int mapTotalHeight;
	private int collisionLayerId;
	
	protected HashMap<String, ArrayList<GroupObject>>	Objects		= new HashMap<String, ArrayList<GroupObject>>();
	protected int layerBelowPlayer = 0;
	/**marks the spots (in tile x,y) which element, if any, is associated with that Tile */
	public HashMap<Vector2f,String>		elementalTilePos = new HashMap<Vector2f, String>();
	public static GameMap CurrentMap;	//there should only be 1 Map
	
	public GameMap(String ref) throws SlickException{
		super(ref);
		this.setMapTotalHeight(this.height* this.tileHeight);
		this.setMapTotalWidth(this.width * this.tileWidth);

		for(int i = 0; i < this.getLayerCount(); i++){
			if(this.getLayerProperty(i, "collision", "0").equals("1")){
				this.collisionLayerId= i;
			}
		}
		/**
		 * parse the Tiles for their elements and save them to the HashMap
		 */
		for(int i = 0; i < this.getLayerCount(); i++){
			for(int x = 0; x < this.getWidth(); x++){
				for(int y = 0; y < this.getHeight(); y++){
					if(this.getTileProperty(this.getTileId(x, y, i), "element", "0") != "0"){
						this.elementalTilePos.put(new Vector2f(x,y), this.getTileProperty(this.getTileId(x, y, i), "element", "0"));
					}
					else if(this.getTileId(x, y, i) != 0){
						//remove Tiles with elements, which are obscurred by other Tiles
						elementalTilePos.remove(new Vector2f(x,y));
					}
				}
			}
		}

				
		for(int i = 0; i < objectGroups.size(); i++){
			System.out.println(this.getObjectCount(i) + " Objekte in Gruppe");
			ObjectGroup curObjGroup = (ObjectGroup) objectGroups.get(i);
			for(int u = 0; u < (curObjGroup).objects.size(); u++){
				GroupObject curGroupObject = (GroupObject) curObjGroup.objects.get(u);				
				if(!Objects.containsKey(curGroupObject.type)){
					Objects.put(curGroupObject.type, new ArrayList<GroupObject>());
					System.out.println("found new Object Group : " + curGroupObject.type);
				}

				Objects.get(curGroupObject.type).add(curGroupObject);
			}
		}
		CurrentMap = this;
		layerBelowPlayer = Integer.parseInt(this.getMapProperty("layerbelowplayer", "0"));
	}
	
	public int getMapTotalWidth() {
		return mapTotalWidth;
	}

	protected void setMapTotalWidth(int mapTotalWidth) {
		this.mapTotalWidth = mapTotalWidth;
	}

	public int getMapTotalHeight() {
		return mapTotalHeight;
	}

	protected void setMapTotalHeight(int mapTotalHeight) {
		this.mapTotalHeight = mapTotalHeight;
	}
	
	public Vector2f getRandPointInSpawnPoint(int index){
		Random Generator = new Random();
		int X = Objects.get("SpawnPoint").get(index).x + Generator.nextInt(Objects.get("SpawnPoint").get(index).width);
		int Y = Objects.get("SpawnPoint").get(index).y + Generator.nextInt(Objects.get("SpawnPoint").get(index).height);
		return new Vector2f(X,Y);
	}
	
	public boolean isPointInObject(String objectName, int index, int X, int Y){
		System.out.println("checking if " + X + "/" + Y + " is in " + Objects.get(objectName).get(index).x + "/" + Objects.get(objectName).get(index).y + " width: " + Objects.get(objectName).get(index).width + " height: " + Objects.get(objectName).get(index).height);
		return (X >= Objects.get(objectName).get(index).x && X <= Objects.get(objectName).get(index).x+Objects.get(objectName).get(index).width && 
				Y >= Objects.get(objectName).get(index).y && Y <= Objects.get(objectName).get(index).y+Objects.get(objectName).get(index).height);
	}
	
	public Vector2f getRandPointInEnemySpawn(int index){
		Random Generator = new Random();
		int X = Objects.get("EnemySpawn").get(index).x + Generator.nextInt(Objects.get("EnemySpawn").get(index).width);
		int Y = Objects.get("EnemySpawn").get(index).y + Generator.nextInt(Objects.get("EnemySpawn").get(index).height);
		return new Vector2f(X,Y);
	}
	
	public Vector2f getRandPointInObject(String ObjectName, int index){
		Random Generator = new Random();
		int X = Objects.get(ObjectName).get(index).x + Generator.nextInt(Objects.get(ObjectName).get(index).width);
		int Y = Objects.get(ObjectName).get(index).y + Generator.nextInt(Objects.get(ObjectName).get(index).height);
		return new Vector2f(X,Y);
	}
	
	public int getNumSpawnPoints(){
		return Objects.get("SpawnPoint").size();
	}
	
	public int getNumEnemySpawns(){
		return Objects.get("EnemySpawn").size();
	}

	//TODO change everything to this Method
	public int getNumObject(String objType){
		return Objects.get(objType).size();
	}

	@Override
	public boolean blocked(PathFindingContext ctx, int x, int y) {
		//TODO: this isn't the true way!!!
		if(x >= this.getWidth()-1 || y >= this.getHeight()-1 || x== 0 || y==0){
			return true; //the	outher Tiles are off-limits
		}
		
		return ((this.getTileId(x, y, collisionLayerId) != 0)|| (x!=0)?(this.getTileId(x-1, y, collisionLayerId) != 0):false ||
				(this.getTileId(x+1, y, collisionLayerId) != 0)||(y!=0)?(this.getTileId(x, y-1, collisionLayerId) != 0):false ||
				(this.getTileId(x, y+1, collisionLayerId) != 0));
	}

	@Override
	public float getCost(PathFindingContext arg0, int arg1, int arg2) {
		return 1.0f;
	}

	@Override
	public int getHeightInTiles() {
		return this.getHeight();
	}

	@Override
	public int getWidthInTiles() {
		return this.getWidth();
	}

	@Override
	public void pathFinderVisited(int arg0, int arg1) {		
	}

	public int getLayerBelowPlayer() {
		return layerBelowPlayer;
	}

	public int getCollisionLayerId() {
		return collisionLayerId;
	}
}
