/***************************************************************************
* A(lchemist) Game, a game about taking elements from the surrounding and throwing to a your enemies.
* Copyright (C) 2012  Sven Stamm
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
***************************************************************************/

package arakash.lpc.base;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.StateBasedGame;

import de.matthiasmann.twl.Button;
import de.matthiasmann.twl.Label;
import de.matthiasmann.twl.ListBox;
import de.matthiasmann.twl.model.SimpleChangableListModel;

import TWLSlick.BasicTWLGameState;
import TWLSlick.RootPane;

/**
 * The scorescreen when a Game is over
 * @author sven
 */
public class GameOverState extends BasicTWLGameState {
    private StateBasedGame game;

    public static final int ID = 30;

    ListBox<String>	yourStats;
    ListBox<String> otherStats;
    ListBox<String> survivors;
    
    Label	thanksLabel;
    Button	okButton;
    
    private Image	background;
    
    @Override
    protected void layoutRootPane() {
    	yourStats.setPosition(20, 300);
    	yourStats.setSize(400, 250);
    	
    	otherStats.setPosition(550, 300);
    	otherStats.setSize(400, 250);
    	
    	survivors.setPosition(250, 100);
    	survivors.setSize(500, 150);
    	
    	thanksLabel.setPosition(400, 600);
    	thanksLabel.adjustSize();
    	
    	okButton.setPosition(thanksLabel.getX(), thanksLabel.getY()+thanksLabel.getHeight()+10);
    	okButton.adjustSize();
    	okButton.addCallback(new Runnable() {
			@Override
			public void run() {
		    	MyGame.getClient().close();
				game.enterState(1);
			}
		});
    }
    
    @Override
    protected RootPane createRootPane() {
        RootPane rp = super.createRootPane();
        rp.setTheme("myTheme");
        yourStats = new ListBox<String>(new SimpleChangableListModel<String>());
        yourStats.setTheme("TextField");
        rp.add(yourStats);
        
        otherStats= new ListBox<String>(new SimpleChangableListModel<String>());
        otherStats.setTheme("TextField");
        rp.add(otherStats);
        
        survivors = new ListBox<String>(new SimpleChangableListModel<String>());
        survivors.setTheme("TextField");
        rp.add(survivors);
        
        thanksLabel = new Label("Thank you for playing!");
        thanksLabel.setTheme("TextField");
        rp.add(thanksLabel);
        
        okButton = new Button("back to Main Menu");
        okButton.setTheme("myButton");
        rp.add(okButton);
        return rp;
    }
    
	@Override
	public void enter(GameContainer arg0, StateBasedGame arg1)
			throws SlickException {
		super.enter(arg0, arg1);
		SimpleChangableListModel<String> model = ((SimpleChangableListModel<String>)otherStats.getModel());
		model.addElement("best Score: " +MyGame.props.get("bestScore")+ " by " + MyGame.props.get("bestScoreName"));
		model.addElement("most Deaths: " +MyGame.props.get("mostDeaths")+ " by "+ MyGame.props.get("mostDeathsName"));
		model.addElement("most Kills: " +MyGame.props.get("mostKills") + " by " + MyGame.props.get("mostKillsName"));
		model.addElement("most Quests: " + MyGame.props.get("mostQuestsCompleted") + " by " + MyGame.props.get("mostQuestsCompletedName"));
		
		model = ((SimpleChangableListModel<String>) yourStats.getModel());
		model.addElement("your Score: " + MyGame.props.get("myScore"));
		model.addElement("your Deaths: "+ MyGame.props.get("myDeaths"));
		model.addElement("your Kills: " + MyGame.props.get("myKills"));
		model.addElement("your Quests Completed: " + MyGame.props.get("myQuestsCompleted"));
	
		model = ((SimpleChangableListModel<String>) survivors.getModel());
		model.addElement(MyGame.props.get("alivePlayers"));
		model.addElement(" made it out of the Armageddon alive.");
		model.addElement(MyGame.props.get("deadPlayers") + " were annihilated.");
	}
    
	@Override
	public void init(GameContainer arg0, StateBasedGame arg1)
			throws SlickException {
		background = new Image("img/NetworkBg.png");
		game=arg1;
	}

	@Override
	public void render(GameContainer arg0, StateBasedGame arg1, Graphics arg2)
			throws SlickException {
		background.draw(0,0);
	}

	@Override
	public void update(GameContainer arg0, StateBasedGame arg1, int arg2)
			throws SlickException {
		// TODO Auto-generated method stub
		
	}

	@Override
	public int getID() {
		return ID;
	}  
}
