/***************************************************************************
* A(lchemist) Game, a game about taking elements from the surrounding and throwing to a your enemies.
* Copyright (C) 2012  Sven Stamm
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
***************************************************************************/

package arakash.lpc.base;

import java.io.File;
import java.io.IOException;
import java.util.Hashtable;
import java.util.List;

import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;
import org.newdawn.slick.SpriteSheet;
import org.newdawn.slick.geom.Rectangle;
import org.newdawn.slick.geom.Shape;
import org.newdawn.slick.geom.Vector2f;

import arakash.lpc.components.DebuffonTouchComponent;
import arakash.lpc.components.DestroyOnDeathComponent;
import arakash.lpc.components.EnemyMovementComponent;
import arakash.lpc.components.LethalTouchComponent;
import arakash.lpc.components.selfdestructComponent;
import arakash.lpc.components.HitBox.HitBoxType;
import arakash.lpc.entity.Entity;
import arakash.lpc.entity.EntityList;
import arakash.lpc.network.server.GameServer;
import arakash.lpc.network.shared.EntityUpdate;

/**
 * this is the Monster and Magic Manager
 * @author sven
 *
 */

public class MMManager {
	Hashtable<String, Hashtable<String, String>> enemyProperties;
	Hashtable<String, Hashtable<String, String>> magicProperties;
	
	static MMManager instance;
	
	public MMManager(){
		enemyProperties = new Hashtable<String, Hashtable<String,String>>();
		magicProperties = new Hashtable<String, Hashtable<String,String>>();
	}
	
	public static MMManager getInstance(){
		return instance;
	}
	
	/**
	 * get all the Properties from the mms.xml
	 */
	public void loadProperties(){
		  SAXBuilder builder = new SAXBuilder();
		  try {
				Document document = builder.build(new File("mms.xml"));
				Element rootNode = document.getRootElement();
				List<Element> list = rootNode.getChildren("enemy");
		 
				for (int i = 0; i < list.size(); i++) {
				   Element node = list.get(i);
				   String enemyName = node.getChildText("name");
				   System.out.println("adding " + enemyName + " enemy");
				   enemyProperties.put(enemyName, new Hashtable<String, String>());
				   enemyProperties.get(enemyName).put("walkingAnim",node.getChildText("walkingAnim"));
				   enemyProperties.get(enemyName).put("deathAnim",node.getChildText("deathAnim"));
				   enemyProperties.get(enemyName).put("speed", node.getChildText("speed"));
				   enemyProperties.get(enemyName).put("maxhealth",node.getChildText("maxhealth"));
				   enemyProperties.get(enemyName).put("flying",node.getChildText("flying"));
				   enemyProperties.get(enemyName).put("awarenessRadius",node.getChildText("awarenessRadius"));
				   enemyProperties.get(enemyName).put("hbOffsetX", node.getChildText("hbOffsetX"));
				   enemyProperties.get(enemyName).put("hbOffsetY", node.getChildText("hbOffsetY"));
				   if(node.getChildText("hbHeight") != null)
				   enemyProperties.get(enemyName).put("hbHeight" , node.getChildText("hbHeight"));
				   if(node.getChildText("hbWidth") != null)
				   enemyProperties.get(enemyName).put("hbWidth" , node.getChildText("hbWidth"));
				}
				
				list = rootNode.getChildren("magic");				 
				for (int i = 0; i < list.size(); i++) {
					Element node = list.get(i);
					String magicName = node.getChildText("name");
					magicProperties.put(magicName, new Hashtable<String, String>());
					magicProperties.get(magicName).put("anim", node.getChildText("anim"));
					magicProperties.get(magicName).put("hbOffsetX", node.getChildText("hbOffsetX"));
					magicProperties.get(magicName).put("hbOffsetY", node.getChildText("hbOffsetY"));
					magicProperties.get(magicName).put("dmgSpriteOffset", node.getChildText("dmgSpriteOffset"));
					magicProperties.get(magicName).put("dmgSpriteEndOffset", node.getChildText("dmgSpriteEndOffset"));
					magicProperties.get(magicName).put("damage", node.getChildText("damage"));
					if(node.getChild("hbHeight") != null)
						magicProperties.get(magicName).put("hbHeight", node.getChildText("hbHeight"));
					if(node.getChild("hbWidth") != null)
						magicProperties.get(magicName).put("hbWidth", node.getChildText("hbWidth"));
					if(node.getChild("effect") != null){
						magicProperties.get(magicName).put("effect", node.getChildText("effect"));
						magicProperties.get(magicName).put("effectAmount", node.getChildText("effectAmount"));
						magicProperties.get(magicName).put("effectDuration", node.getChildText("effectDuration"));
					}
					if(node.getChild("sound") != null)
						magicProperties.get(magicName).put("sound", node.getChildText("sound"));
				} 
		  } catch (IOException io) {
				System.out.println(io.getMessage());
		  } catch (JDOMException jdomex) {
			System.out.println(jdomex.getMessage());
		  } catch (NumberFormatException e) {
			e.printStackTrace();
		}
	}
	
	public Entity generateMeAMagic(String magicName, int centerPosX, int centerPosY){
		Entity e = Entity.createNewEntity();
		Hashtable<String,String> properties = magicProperties.get(magicName);
		LPCAnimation anim = ResourceManager.Animations.get(properties.get("anim"));
		e.addComponent(new selfdestructComponent(anim.getTimeForOneAnim()*anim.getSpriteSheet().getHorizontalCount()));
		SpriteSheetPlus ssp = anim.getSpriteSheet();
		e.width = ssp.getSpriteWidth();
		e.height= ssp.getSpriteHeight();
		int offsetX = Integer.parseInt(properties.get("hbOffsetX"));
		int offsetY = Integer.parseInt(properties.get("hbOffsetY"));
		int hbHeight= 0;
		int hbWidth = 0;
		if(properties.get("hbHeight") != null)
			hbHeight = Integer.parseInt(properties.get("hbHeight"));
		else
			hbHeight = e.height-2*offsetY;
		if(properties.get("hbWidth") != null)
			hbWidth = Integer.parseInt(properties.get("hbWidth"));
		else
			hbWidth = e.width-2*offsetX;
		Shape hitShape = new Rectangle(e.getPosition().x, e.getPosition().y, hbWidth, hbHeight);

		/**
		 * currently the time Offset doesnt do anything...
		 */
		int dmgSpriteOffset = Integer.parseInt(properties.get("dmgSpriteOffset"));
		int dmgSpriteEndOffset= Integer.parseInt(properties.get("dmgSpriteEndOffset"));
		int damage = Integer.parseInt(properties.get("damage"));
		if(properties.get("effect") != null){ //does the Magic have an effect on it? (like slow, or on fire, or pushback
			//effect Properties are very unique, so handle them individual
			//these effects should go in an extra Component, as they are not directly part of the "damage" component
			int Time = Integer.parseInt(properties.get("effectDuration"));
			if(properties.get("effect").equals("slow")){
				float slowed = (float) Integer.parseInt(properties.get("effectAmount"))/100;
				System.out.println("slowed amount: " + slowed);
				e.addComponent(new DebuffonTouchComponent(hitShape, offsetX, offsetY, HitBoxType.ENEMY, "speed", slowed,"slow", Time));
				e.addComponent(new DebuffonTouchComponent(hitShape, offsetX, offsetY, HitBoxType.PLAYER, "speed", slowed,"slow", Time));
			}
			else if(properties.get("effect").equals("pushback")){
				e.addComponent(new DebuffonTouchComponent(hitShape, offsetX, offsetX, HitBoxType.ENEMY, "speed",-1, "pushback", Time));
				e.addComponent(new DebuffonTouchComponent(hitShape, offsetX, offsetX, HitBoxType.PLAYER, "speed",-1, "pushback", Time));
			}
		}
		e.addComponent(new LethalTouchComponent(hitShape, offsetX, offsetY, HitBoxType.ENEMY, damage));
		
		e.setDIR((short) 0);
		e.setOrAddProperty("resourceName", properties.get("anim"));
		if(properties.get("sound")!=null)
		e.setOrAddProperty("soundResource", properties.get("sound"));
		
		EntityUpdate eu = new EntityUpdate();
		e.setPosition(new Vector2f(centerPosX-e.width/2, 
				centerPosY-e.height/2));
		eu.setResourceName(properties.get("anim"));
		eu.setEntity(e);
		eu.setSelfdestructTimer(anim.getTimeForOneAnim()*anim.getSpriteSheet().getHorizontalCount());
		GameServer.addUpdate(eu);
		return e;
	}
	
	public Entity generateMeAEnemy(String enemyName){
		System.out.println("getting properties for enemy: " + enemyName);
		Hashtable<String,String> properties = enemyProperties.get(enemyName);
		if(properties == null)
		System.out.println("properties are null!");
		
		Entity enemy = Entity.createNewEntity();
		enemy.setType(EntityList.ENEMY_TYPE);		//hm, i dont know how to feel about this, maybe an extra "EnemyUpdate"?
		enemy.setOrAddProperty("speed", properties.get("speed"));
		enemy.setOrAddProperty("health", properties.get("maxhealth"));
		enemy.setOrAddProperty("damage", "0");
		enemy.setOrAddProperty("maxhealth",properties.get("maxhealth"));
		enemy.setOrAddProperty("name", enemyName);
		System.out.println("setting resourceName " + properties.get("walkingAnim") + " on " + enemy.getID());
		enemy.setOrAddProperty("resourceName", properties.get("walkingAnim"));
		enemy.setOrAddProperty("flying", properties.get("flying"));
//		enemy.setOrAddProperty("deathAnim", properties.get("deathAnim"));
		enemy.width = ((SpriteSheet) ResourceManager.Resources.get("slimeEnemy")).getSprite(0, 0).getWidth();
		enemy.height= ((SpriteSheet) ResourceManager.Resources.get("slimeEnemy")).getSprite(0, 0).getHeight();
		int offsetX = Integer.parseInt(properties.get("hbOffsetX"));
		int offsetY = Integer.parseInt(properties.get("hbOffsetY"));
		int hbHeight= 0;
		int hbWidth = 0;
		if(properties.get("hbHeight") != null)
			hbHeight = Integer.parseInt(properties.get("hbHeight"));
		else
			hbHeight = enemy.height-2*offsetY;
		if(properties.get("hbWidth") != null)
			hbWidth = Integer.parseInt(properties.get("hbWidth"));
		else
			hbWidth = enemy.width-2*offsetX;
		Shape enemyShape = new Rectangle(enemy.getPosition().x, enemy.getPosition().y, hbWidth, hbHeight);
		int awarenessRadius = Integer.parseInt(properties.get("awarenessRadius"));
		enemy.addComponent(new EnemyMovementComponent(enemyShape, HitBoxType.ENEMY, offsetX, offsetY, awarenessRadius, 150));
		enemy.addComponent(new LethalTouchComponent(enemyShape, offsetX, offsetY, HitBoxType.PLAYER,200));	
		enemy.addComponent(new DestroyOnDeathComponent(properties.get("deathAnim")));
		return enemy;
	}
	
	public static MMManager createNewManager(){
		instance = new MMManager();
		instance.loadProperties();
		return instance;
	}
}
