/***************************************************************************
* A(lchemist) Game, a game about taking elements from the surrounding and throwing to a your enemies.
* Copyright (C) 2012  Sven Stamm
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
***************************************************************************/
package arakash.lpc.base;

import org.newdawn.slick.geom.Rectangle;
import org.newdawn.slick.geom.Shape;
import org.newdawn.slick.geom.Vector2f;

import arakash.lpc.Map.GameMap;
import arakash.lpc.components.HitBox;
import arakash.lpc.components.NPCPingComponent;
import arakash.lpc.components.PropertyChangeComponent;
import arakash.lpc.components.QuestGiverComponen;
import arakash.lpc.components.QuestTargetComponent;
import arakash.lpc.components.scrollPickupComponent;
import arakash.lpc.components.selfdestructComponent;
import arakash.lpc.components.touchComponent;
import arakash.lpc.components.HitBox.HitBoxType;
import arakash.lpc.entity.Entity;
import arakash.lpc.entity.EntityManager;
import arakash.lpc.network.server.GameServer;
import arakash.lpc.network.server.PropertyChangeUpdate;
import arakash.lpc.network.server.SoundUpdate;
import arakash.lpc.network.shared.EntityUpdate;
import arakash.lpc.network.shared.MessageUpdate;

/**
 * stores and manages all the Quests.
 * Quest 
 * @author sven
 *
 */
public class QuestManager {
	static int QuestID[];
	static String QuestShortText[];
	static String QuestFullText[];
		
	public static void initTestQuestManager(){
		QuestID = new int[2];
		QuestShortText = new String[2];
		QuestFullText = new String[2];
		
		//******* testQuest
		QuestID[0] = 1;
		QuestShortText[0]= "Save the Princess!";
		QuestFullText[0] = "the Princess has been lost. Save her!"; 
		
		QuestID[1] = 5;
		QuestShortText[1] = "go on a walk with his bee!";
		QuestFullText[1] = "My Bee got Fat, would you go on a walk with her?";
	}
	
	public static String getShortText(int QuestID){
		for(int i = 0; i < QuestShortText.length; i++){
			if(QuestManager.QuestID[i] == QuestID){
				return QuestShortText[i];
			}
		}
		return null;
	}
	
	public static String getFullText(int QuestID){
		for(int i = 0; i < QuestFullText.length; i++){
			if(QuestManager.QuestID[i] == QuestID){
				return QuestFullText[i];
			}
		}
		return null;
	}
	
	public static void generateNPCs(){
		   /***** 
		    * test NPC
		    */
		   Entity NPC = Entity.createNewEntity();
		   NPC.addComponent(new QuestGiverComponen(1));
		   NPC.setPosition(new Vector2f(GameMap.CurrentMap.getRandPointInObject("NPCSpawn", GameServer.randGenerator.nextInt(GameMap.CurrentMap.getNumObject("NPCSpawn")))));
		   NPC.setDIR((short)2);
		   NPC.setOrAddProperty("resourceName", "maleSoldier");
		   EntityManager.curManager.Entities.add(NPC);
		   EntityUpdate eu = new EntityUpdate();
		   eu.setEntity(NPC);
		   eu.setResourceName("maleSoldier");
		   GameServer.addUpdate(eu);
		   NPC = Entity.createNewEntity();
		   NPC.addComponent(new QuestTargetComponent(1) {						
			@Override
			public void onQuestComplete(Entity player) {
				MessageUpdate mu = new MessageUpdate();
				mu.setMsg("Thank you!");
				mu.setOwnerID(owner.getID());
				GameServer.addUpdate(mu);
				player.setOrAddProperty("currentQuest", "");
				int numQuestsCompleted = player.getProperty("questsCompleted")!=null?Integer.parseInt(player.getProperty("questsCompleted")):0;
				player.setOrAddProperty("questsCompleted", ""+(numQuestsCompleted+1));
				PropertyChangeUpdate pcu = new PropertyChangeUpdate();
				pcu.setEntityID(player.getID());
				pcu.setPropertyName(new String[]{"currentQuest", "score", "questsCompleted"});
				int CurScore = player.getProperty("score") != null ? Integer.parseInt(player.getProperty("score")) : 0;
				pcu.setPropertyValue(new String[]{"", ""+(CurScore + 100), player.getProperty("questsCompleted")});
				player.setOrAddProperty("score", ""+(CurScore+100));
				GameServer.addUpdate(pcu);
				mu = new MessageUpdate();
				mu.setOwnerID(-1);
				mu.setMsg(player.getProperty("name") + " has found the princess!");
				GameServer.addUpdate(mu);
				
				int scroll = GameServer.randGenerator.nextInt(3);
				Entity e = Entity.createNewEntity();
				SpriteSheetPlus ssp = (SpriteSheetPlus) ResourceManager.Resources.get("elementalScroll");
				Shape scrollShape = new Rectangle(0, 0, ssp.getSpriteWidth(), ssp.getSpriteHeight());
				//TODO: well, technically this would fall into the Pickup class, but i dont see it beneficial here, probably i can just cut it all together
				e.addComponent(new scrollPickupComponent(scrollShape, 0, 0, HitBoxType.PLAYER, scroll));
				e.setDIR((short) scroll);
				e.setOrAddProperty("resourceName", "elementalScroll");
				e.setPosition(owner.getPosition());
				EntityManager.curManager.registerNewEntity(e);
				//send it to da playas! 
				EntityUpdate eu = new EntityUpdate();
				eu.setEntity(e);
				eu.setResourceName("elementalScroll");
				GameServer.addUpdate(eu);
				
				//owner.addComponent(new EnemyMovementComponent(new Rectangle(0, 0, 10, 10), null, 0 , 16, 1000, 50));
				owner.setPosition(new Vector2f(GameMap.CurrentMap.getRandPointInEnemySpawn(GameServer.randGenerator.nextInt(GameMap.CurrentMap.getNumEnemySpawns()))));
			}
		});
		   NPC.setPosition(new Vector2f(GameMap.CurrentMap.getRandPointInEnemySpawn(GameServer.randGenerator.nextInt(GameMap.CurrentMap.getNumEnemySpawns()))));
		   NPC.setOrAddProperty("resourceName", "princess");
		   //NPC.setOrAddProperty("speed", "20");
		   NPC.setDIR((short)2);
		   EntityManager.curManager.Entities.add(NPC);
		   eu = new EntityUpdate();
		   eu.setEntity(NPC);
		   eu.setResourceName("princess");
		   GameServer.addUpdate(eu);
		   /**
		    * End Test NPC
		    */
	}
	
	public static void generateExkalibur(){
		   /**
		    * test Excalibur
		    */
		   	Entity excal = Entity.createNewEntity();
		   	excal.setPosition(GameMap.CurrentMap.getRandPointInObject("ExcaliburSpawn", 0));
		   	excal.setOrAddProperty("resourceName", "goldenSword");
		   	excal.addComponent(new touchComponent(new Rectangle(0, 0, 40, 40),HitBoxType.PICKUP, 0,0, HitBoxType.PLAYER) {
				@Override
				public void onTouch(HitBox touchedHB, int delta) {
					Entity toucher = touchedHB.getOwnerEntity();
					if(owner.getProperty("touched") != null && owner.getProperty("touched").contains(toucher.getID()+"#"))
						return;
					
					owner.addComponent(new PropertyChangeComponent(3000, "touched", ""+toucher.getID()+"#", PropertyChangeComponent.PROP_CONCAT));
					if(toucher.isPropertyZeroOrNull("firelvl") || toucher.isPropertyZeroOrNull("earthlvl") || toucher.isPropertyZeroOrNull("waterlvl") ||
							Integer.parseInt(toucher.getProperty("firelvl"))<2 || Integer.parseInt(toucher.getProperty("earthlvl")) < 2 || Integer.parseInt(toucher.getProperty("waterlvl"))<2){
						MessageUpdate mu = new MessageUpdate();
						mu.setOwnerID(owner.getID());
						mu.setMsg("You are not yet worthy of me!");
						GameServer.addUpdate(mu);
					}
					else{
						MessageUpdate mu = new MessageUpdate();
						mu.setOwnerID(owner.getID());
						mu.setMsg("You fool! Brace yourself for the Armageddon!");
						GameServer.addUpdate(mu);
						GameServer.armageddon = true;
						owner.addComponent(new selfdestructComponent(2500));
						
						int newScore = toucher.getProperty("score")!=null? (Integer.parseInt(toucher.getProperty("score"))+500):500;
						toucher.setOrAddProperty("score",""+newScore);
						PropertyChangeUpdate pcu = new PropertyChangeUpdate();
						pcu.setEntityID(toucher.getID());
						pcu.setPropertyName(new String[]{"score"}); pcu.setPropertyValue(new String[]{""+newScore});
						GameServer.addUpdate(pcu);
						pcu = new PropertyChangeUpdate();
						pcu.setEntityID(owner.getID());
						pcu.setPropertyName(new String[]{"selfdestruct"}); pcu.setPropertyValue(new String[]{"2500"});
						GameServer.addUpdate(pcu);
						
						mu = new MessageUpdate();
						mu.setOwnerID(-1);
						mu.setMsg(toucher.getProperty("name") + " HAS STARTED THE ARMAGEDDON!");
						GameServer.addUpdate(mu);
						SoundUpdate su = new SoundUpdate("theDarkAmulet", true);
						su.setBackgroundMusic(true);
	 					GameServer.addUpdate(su);
	 					
	 					/**
	 					 * create Guy who pings the Location of the SafeZone
	 					 */
	 					Entity e = Entity.createNewEntity();
	 					e.setOrAddProperty("resourceName", "maleSoldier");
	 					e.setPosition(new Vector2f(GameMap.CurrentMap.getRandPointInObject("SafePoint", 0)));
	 					e.addComponent(new NPCPingComponent(3000));
	 					e.setDIR((short)1);
	 					EntityManager.curManager.Entities.add(e);
	 					
	 					EntityUpdate eu = new EntityUpdate();
	 					eu.setEntity(e);
	 					GameServer.addUpdate(eu);
					}
				}
			});
		   	EntityManager.curManager.Entities.add(excal);
		   //end test Excalibur
	}
}
