/***************************************************************************
* A(lchemist) Game, a game about taking elements from the surrounding and throwing to a your enemies.
* Copyright (C) 2012  Sven Stamm
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
***************************************************************************/
package arakash.lpc.base;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;

import org.newdawn.slick.Image;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.Sound;

import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.JDOMException;
import org.jdom2.input.SAXBuilder;
import arakash.lpc.base.LPCAnimation;

public class ResourceManager{
	public static HashMap<String, Image>	Resources = new HashMap<String, Image>();
	
	public static HashMap<String, LPCAnimation> Animations= new HashMap<String, LPCAnimation>();
	
	public static HashMap<String, Sound>	Sounds = new HashMap<String, Sound>();
	public static Sound		backgroundMusic;
	
	public static void registerNewImage(String name, Image image){
		System.out.println("registering new Image: " + name);
		Resources.put(name, image);
	}
	
	public static synchronized void init(){
		  SAXBuilder builder = new SAXBuilder();
		  if(Resources.size() > 0 || Animations.size() > 0 || Sounds.size() > 0){
			  System.out.println("WARNING: ResourceManager already has resources loaded! skipping.");
		  }
		  try {
				Document document = builder.build(new File("resources.xml"));
				Element rootNode = document.getRootElement();
				List<Element> list = rootNode.getChildren("spriteSheet");
		 
				for (int i = 0; i < list.size(); i++) {
				   Element node = list.get(i);
				   SpriteSheetPlus ss = new SpriteSheetPlus(node.getChildText("src"), 
						   				Integer.parseInt(node.getChildText("spritewidth")), Integer.parseInt(node.getChildText("spriteheight")));
				   registerNewImage(node.getChildText("name"), ss);
				}
		 
				list = rootNode.getChildren("animation");
				for(int i = 0; i < list.size(); i++){
					Element node = list.get(i);
					String sourceName = node.getChildText("ssSrc");
					int timeForOneAnim = Integer.parseInt(node.getChildText("timeForOneAnim"));
					boolean hasStandingSprite = Boolean.parseBoolean(node.getChildText("hasStandingSprite"));
					LPCAnimation lpca = new LPCAnimation((SpriteSheetPlus)Resources.get(sourceName), 
											timeForOneAnim, 
											hasStandingSprite);
					if(node.getChildText("norepeat")!= null)
						lpca.setNorepeat(Boolean.parseBoolean(node.getChildText("norepeat")));
					Animations.put(sourceName, lpca);
				}
				list = rootNode.getChildren("sound");
				for(int i = 0; i < list.size(); i++){
					Element node = list.get(i);
					String name = node.getChildText("name");
					String src = node.getChildText("src");
					Sounds.put(name, new Sound(src));
				}
			  } catch (IOException io) {
				System.out.println(io.getMessage());
			  } catch (JDOMException jdomex) {
				System.out.println(jdomex.getMessage());
			  } catch (NumberFormatException e) {
				e.printStackTrace();
			} catch (SlickException e) {
				e.printStackTrace();
			}
	}
	
	public static void loopSound(String name){
		if(!MyGame.Options.getProperty("sound").equals("off")){
			Sounds.get(name).loop();
		}
	}
	
	public static void playSound(String name){
		if(!MyGame.Options.getProperty("sound").equals("off")){
			Sounds.get(name).play();
		}
	}
	
	public static LPCAnimation getAnimationFromResName(String resourceName){
		if(Animations.get(resourceName) != null)
			return ResourceManager.Animations.get(resourceName);
		else
			return new LPCAnimation((SpriteSheetPlus) Resources.get(resourceName), 0, false);
	}
	
	public static void setBackgroundMusic(String soundName){
		if(backgroundMusic != null)
			backgroundMusic.stop();
		backgroundMusic = Sounds.get(soundName);
		loopSound(soundName);
	}
	
/*	public static void clearAllResources(){
		
	}*/
}
