/***************************************************************************
* A(lchemist) Game, a game about taking elements from the surrounding and throwing to a your enemies.
* Copyright (C) 2012  Sven Stamm
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
***************************************************************************/
package arakash.lpc.components;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.state.StateBasedGame;

import com.esotericsoftware.kryonet.rmi.ObjectSpace;
import com.esotericsoftware.kryonet.rmi.RemoteObject;

import arakash.lpc.base.EntityManagerInterface;
import arakash.lpc.base.LPCAnimation;
import arakash.lpc.base.ResourceManager;
import arakash.lpc.entity.Entity;
import arakash.lpc.entity.EntityManager;
import arakash.lpc.network.server.GameServer;
import arakash.lpc.network.server.PlayerClient;
import arakash.lpc.network.server.PropertyChangeUpdate;
import arakash.lpc.network.shared.EntityUpdate;

/**
 * destroys the Entity and all Components and removes it from the EntityManager when its heath is below 0
 * used Properties from Entity:
 * health, damage
 * @author sven
 *
 */
public class DestroyOnDeathComponent extends Component{

	String deathResourceName;
	
	public DestroyOnDeathComponent(String deathResourceName) {
		this(ComponentList.DESTROY_ON_DEATH_COMPONENT, deathResourceName);
	}
	
	public DestroyOnDeathComponent(int type, String deathResourceName) {
		super(type);
		this.deathResourceName = deathResourceName;
	}

	@Override
	public void update(GameContainer gc, StateBasedGame sbgame, int delta) {
		if(owner.getProperty("health") != null && owner.getProperty("damage") != null){
			int health = Integer.parseInt(owner.getProperty("health")) - Integer.parseInt(owner.getProperty("damage"));
			owner.setOrAddProperty("health", "" + health);
			owner.setOrAddProperty("damage", "0");
		}
		if(owner.getProperty("damage")!= null && owner.getProperty("health")!=null && 
				Integer.parseInt(owner.getProperty("health")) <= 0){
			EntityManager.curManager.deregisterEntity(owner.getID());
			for(PlayerClient pc : GameServer.clients){
				EntityManagerInterface em = ObjectSpace.getRemoteObject(pc.getClientConnection(), 42, EntityManagerInterface.class);
				((RemoteObject) em).setNonBlocking(true, true);
				em.deregisterEntity(owner.getID());
				System.out.println("done");
			}
			
			if(owner.getProperty("lastHit") != null){
				int lastHitID = Integer.parseInt(owner.getProperty("lastHit"));
				Entity lastHitter = null;
				for(int i = 0; i < GameServer.clients.size(); i++){
					if(lastHitID == GameServer.clients.get(i).getClientEntity().getID()){
						lastHitter = GameServer.clients.get(i).getClientEntity();
						break;
					}
				}
				if(lastHitter != null){
					//give the player Points for his kill!
					int curScore = lastHitter.getProperty("score")!=null? Integer.parseInt(lastHitter.getProperty("score")) : 0;
					lastHitter.setOrAddProperty("score", ""+(curScore+10));
					//increase his killcount by 1
					int curKills = lastHitter.getProperty("kills")!=null? Integer.parseInt(lastHitter.getProperty("kills")) : 0;
					lastHitter.setOrAddProperty("kills",""+ (curKills+1));
					PropertyChangeUpdate pcu = new PropertyChangeUpdate();
					pcu.setEntityID(lastHitID);
					pcu.setPropertyName(new String[]{"score","kills"}); pcu.setPropertyValue(new String[]{""+(curScore+10), ""+ (curKills+1)});
					GameServer.addUpdate(pcu);
				}
			}
			EntityUpdate eu = new EntityUpdate();
			Entity DeathEntity = Entity.createNewEntity();
			DeathEntity.setPosition(owner.getPosition());
			eu.setEntity(DeathEntity);
			eu.setResourceName(deathResourceName);				
			LPCAnimation anim = ResourceManager.Animations.get(deathResourceName);
			eu.setSelfdestructTimer(anim.getTimeForOneAnim()*anim.getSpriteSheet().getHorizontalCount());
			GameServer.addUpdate(eu);
		}
	}

	@Override
	public void cleanup() {
		// TODO Auto-generated method stub
		
	}
}
