/***************************************************************************
* A(lchemist) Game, a game about taking elements from the surrounding and throwing to a your enemies.
* Copyright (C) 2012  Sven Stamm
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
***************************************************************************/
package arakash.lpc.components;

import java.util.Random;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.geom.Circle;
import org.newdawn.slick.geom.Shape;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.util.pathfinding.AStarPathFinder;
import org.newdawn.slick.util.pathfinding.Path;

import arakash.lpc.Map.GameMap;
import arakash.lpc.entity.Entity;

public class EnemyMovementComponent extends MovementComponent {
	int awarenessRadius;
	int enemyNoticeUpadateMs;
	int updateTimer = 0;
	Shape awarenessShape;
	int TargetTileX = -1;
	int TargetTileY = -1;
	Path pathToEnemy = null;
	int	 pathStep	= 0;
	
	public EnemyMovementComponent(Shape hitboxshape, HitBoxType hitBoxType, int offsetX, int offsetY, int awarenessRadius, int enemyNoticeUpdateMs) {
		this(ComponentList.ENEMY_MOVEMENT_COMPONENT, hitboxshape, hitBoxType, offsetX, offsetY, awarenessRadius, enemyNoticeUpdateMs);
	}
	
	public EnemyMovementComponent(int type, Shape hitboxshape, HitBoxType hitBoxType, int offsetX, int offsetY, int awarenessRadius, int enemyNoticeUpdateMs) {
		super(type, hitboxshape, hitBoxType, offsetX, offsetY);
		this.awarenessRadius = awarenessRadius;
		this.enemyNoticeUpadateMs = enemyNoticeUpdateMs;
		this.awarenessShape = new Circle(0, 0, awarenessRadius);
	}

	@Override
	public void update(GameContainer gc, StateBasedGame sbgame, int delta) {
		//TODO: rework, screws up enemy movement
		this.awarenessShape.setCenterX(owner.getPosition().x +offsetX + (owner.width-offsetX)/2);
		this.awarenessShape.setCenterY(owner.getPosition().y + owner.height/2 + offsetY);
//		this.awarenessShape.setCenterY(this.collisionShape.getCenterY());
		if(updateTimer > enemyNoticeUpadateMs){
				//only once every few hundrerd ms. or even every second, see enemyNoticeUpdateMs
			if(!noticeEnemy()){
				//if no enemy is found, go in a random direction
				Random generator = new Random();
				if(generator.nextInt(4) == 1)	//fewer direction changes.. (but is this alright?)
					owner.setDIR((short)generator.nextInt(4));
				TargetTileX = -1;
				TargetTileY = -1;
				pathToEnemy = null;
				pathStep = 0;
			}
			else if(pathToEnemy == null || (pathToEnemy.getX(pathToEnemy.getLength()-1) != TargetTileX && pathToEnemy.getY(pathToEnemy.getLength()-1) != TargetTileY)){	//enemy was found, doesn't check if the Path is still the Same
				if(owner.getProperty("flying") == null || !owner.getProperty("flying").equals("True"))
						generatePath();
				else{
					pathToEnemy = null;
					int curTileX =(int) this.awarenessShape.getCenterX()/GameMap.CurrentMap.getTileWidth();
					int curTileY =(int) this.awarenessShape.getCenterY()/GameMap.CurrentMap.getTileHeight();
					if(TargetTileX > curTileX){
						owner.setDIR(Entity.DIR_RIGHT);
					}
					else if(TargetTileX < curTileX ){
						owner.setDIR(Entity.DIR_LEFT);
					}
					else if(TargetTileY > curTileY)
						owner.setDIR(Entity.DIR_DOWN);
					else if(TargetTileY < curTileY)
						owner.setDIR(Entity.DIR_UP);
				}
				pathStep = 0;
			}
			updateTimer = 0;
		}
		updateTimer += delta;
		if(pathToEnemy != null){
			int CurTileX = (int)this.awarenessShape.getCenterX()/map.getTileWidth();
			int CurTileY = (int)this.awarenessShape.getCenterY()/map.getTileHeight(); 
			
			if(CurTileX < pathToEnemy.getX(pathStep)){
				owner.setDIR(Entity.DIR_RIGHT);
			}
			else if(CurTileX > pathToEnemy.getX(pathStep)){
				owner.setDIR(Entity.DIR_LEFT);
			}
			else if(CurTileY < pathToEnemy.getY(pathStep)){
				owner.setDIR(Entity.DIR_DOWN);
			}
			else if(CurTileY > pathToEnemy.getY(pathStep)){
				owner.setDIR(Entity.DIR_UP);
			}
			if(CurTileY == pathToEnemy.getY(pathStep) && CurTileX == pathToEnemy.getX(pathStep)){
				pathStep++;
			}
			if(pathToEnemy.getLength() == pathStep){
				pathToEnemy = null;
				pathStep = 0;
				TargetTileX= -1;
				TargetTileY= -1;
			}
		}
		
		super.update(gc, sbgame, delta);
		}

	private boolean noticeEnemy() {
		for(HitBox hb : HitBox.PlayerHitBoxes){
			if(this.awarenessShape.intersects(hb.collisionShape)&& hb.getOwnerEntity().isPropertyZeroOrNull("death") ){
				//System.out.println("awareness intersects!" + hb.ID + " " + hb.collisionShape.getCenterX() + " " + hb.getCollisionShape().getCenterY() + " mine: " + this.collisionShape.getCenterX() + " " + this.collisionShape.getCenterY());
				if(TargetTileX == -1 || TargetTileY == -1){
					TargetTileX = (int) (hb.collisionShape.getCenterX() /map.getTileWidth());
					TargetTileY = (int) (hb.collisionShape.getCenterY() / map.getTileHeight());
					System.out.println("noticed Enemy!!!");
					return true;
				}
					
				else if(getDistanceSquared(TargetTileX*map.getTileWidth(), TargetTileY*map.getTileHeight(), (int) this.awarenessShape.getCenterX(), (int)this.awarenessShape.getCenterY())
				  >getDistanceSquared((int)hb.collisionShape.getCenterX(),(int)hb.collisionShape.getCenterY(), (int) this.awarenessShape.getCenterX(), (int) this.awarenessShape.getCenterY())){
					TargetTileX = (int) (hb.collisionShape.getCenterX() /map.getTileWidth());
					TargetTileY = (int) (hb.collisionShape.getCenterY() / map.getTileHeight());
					System.out.println("noticed new Enemy!");
					return true;
				}
			}
		}
		return false;
	}

	@Override
	public void cleanup() {
		super.cleanup();
	}
	
	private void generatePath(){
		AStarPathFinder pathFinder = new AStarPathFinder(map, 35, false);
		pathToEnemy = pathFinder.findPath(null, (int) (this.awarenessShape.getCenterX()/map.getTileWidth()), (int) (this.awarenessShape.getCenterY()/map.getTileHeight()), 
				TargetTileX, TargetTileY);
	}
	
	public static int getDistanceSquared(int x1, int y1, int x2, int y2){
		return (int) (Math.pow(x1-x2,2)+ Math.pow(y1-y2, 2));
	}
}
