/***************************************************************************
* A(lchemist) Game, a game about taking elements from the surrounding and throwing to a your enemies.
* Copyright (C) 2012  Sven Stamm
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
***************************************************************************/
package arakash.lpc.components;

import java.util.ArrayList;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.geom.Shape;
import org.newdawn.slick.geom.Vector2f;
import org.newdawn.slick.state.StateBasedGame;

import arakash.lpc.Map.GameMap;

public class HitBox extends Component {

	public enum HitBoxType{
		WORLD, PLAYER, ENEMY, PROJECTILE, PICKUP
	}
	
	transient public		static GameMap	map;
	transient public		static int 	collisionLayerId;
	protected 			Shape 	collisionShape;
	
	protected 	static ArrayList<HitBox> WorldHitBoxes = new ArrayList<HitBox>();	//not needed
	public 		static ArrayList<HitBox> PlayerHitBoxes= new ArrayList<HitBox>();
	public 		static ArrayList<HitBox> EnemyHitBoxes = new ArrayList<HitBox>();
	public 		static ArrayList<HitBox> ProjectilesHitBoxes = new ArrayList<HitBox>();
	public	 	static ArrayList<HitBox> PickupHitBoxes	= new ArrayList<HitBox>();
	
	protected int offsetX;
	protected int offsetY;
	protected HitBoxType hbtype;
	
	public HitBox(Shape hitboxshape, HitBoxType type, int offsetX, int offsetY){
		this(ComponentList.HIT_BOX_COMPONENT, hitboxshape, type, offsetX, offsetY);
	}
	
	public HitBox(int compType, Shape hitboxshape, HitBoxType type,int offsetX,int offsetY){
		super(compType);
		
		this.offsetX = offsetX;
		this.offsetY = offsetY;
		this.hbtype = type;
		
		this.collisionShape = hitboxshape;
		if(type == HitBoxType.WORLD)
			WorldHitBoxes.add(this);
		else if(type == HitBoxType.PLAYER)
			PlayerHitBoxes.add(this);
		else if(type == HitBoxType.ENEMY)
			EnemyHitBoxes.add(this);
		else if(type == HitBoxType.PROJECTILE)
			ProjectilesHitBoxes.add(this);
		else if(type == HitBoxType.PICKUP)
			PickupHitBoxes.add(this);
	}
	
	@Override
	public void update(GameContainer gc, StateBasedGame sbgame, int delta) {
		this.collisionShape.setLocation(new Vector2f(this.getOwnerEntity().getPosition().x+offsetX, this.getOwnerEntity().getPosition().y+offsetY));
	}
	
	public Shape getCollisionShape(){
		return collisionShape;
	}

	public void setCollisionShapePos(Vector2f pos){
		this.collisionShape.setLocation(pos);
	}
	
	public boolean WorldCollideDown(){
		float X = (this.collisionShape.getX()) / map.getTileWidth();
		float Y = (this.collisionShape.getY()+this.collisionShape.getHeight()) / map.getTileHeight();
		if(X <= 0 || Y <= 0)
			return true;
		if(map.getTileId((int) (X+this.collisionShape.getWidth()/map.getTileWidth()),(int) Y, collisionLayerId) != 0 ||
				map.getTileId((int) X, (int) Y, collisionLayerId) != 0){
			return true;
			//|| map.getTileId((int) X, (int) Y, collisionLayerId) != 0
		}
		else
			return false;
		
	}
	
	public boolean WorldCollideRight(){
		float X = (this.collisionShape.getX()+this.collisionShape.getWidth()) / map.getTileWidth();
		float Y = (this.collisionShape.getY()) / map.getTileHeight();
		if(map.getTileId((int) X,(int) Y, collisionLayerId) != 0 ||
				map.getTileId((int) X,(int) (Y + this.collisionShape.getHeight()/map.getTileHeight()), collisionLayerId) != 0){
			return true;
		}
		else
			return false;
	}
	
	public boolean WorldCollideLeft(){
		float X = (this.collisionShape.getX()) / map.getTileWidth();
		float Y = (this.collisionShape.getY()) / map.getTileHeight();
		if(map.getTileId((int) X, (int) Y, collisionLayerId) != 0 ||
				map.getTileId((int) X,(int) (Y+this.collisionShape.getHeight()/map.getTileHeight()), collisionLayerId) != 0){
			return true;
		}
		return false;
	}

	public boolean WorldCollideUp(){
		float X = (this.collisionShape.getX()) / map.getTileWidth();
		float Y = (this.collisionShape.getY()) / map.getTileHeight();
		
		if(map.getTileId((int) X, (int) Y, collisionLayerId) != 0 ||
				map.getTileId((int) (X+this.collisionShape.getWidth()/map.getTileWidth()),(int) Y, collisionLayerId) != 0){
			return true;
		}
		return false;
	}

	@Override
	public void cleanup() {
	System.out.println("HitBox cleanup");
	if(this.hbtype == HitBoxType.PLAYER){
		PlayerHitBoxes.remove(this);
		System.out.println("removed Player Hitbox! new Size: " + PlayerHitBoxes.size());
	}
	if(this.hbtype == HitBoxType.ENEMY)
		EnemyHitBoxes.remove(this);
	if(this.hbtype == HitBoxType.PICKUP)
		PickupHitBoxes.remove(this);
	if(this.hbtype == HitBoxType.PROJECTILE)
		ProjectilesHitBoxes.remove(this);
	if(this.hbtype == HitBoxType.WORLD)
		WorldHitBoxes.remove(this);
	}

	public int getOffsetX() {
		return offsetX;
	}

	public void setOffsetX(int offsetX) {
		this.offsetX = offsetX;
	}

	public int getOffsetY() {
		return offsetY;
	}

	public void setOffsetY(int offsetY) {
		this.offsetY = offsetY;
	}
	
	public ArrayList<HitBox> getHitBoxes(HitBoxType type){
		if(type == HitBoxType.WORLD)
			return WorldHitBoxes;
		else if(type == HitBoxType.PLAYER)
			return PlayerHitBoxes;
		else if(type == HitBoxType.ENEMY)
			return EnemyHitBoxes;
		else if(type == HitBoxType.PROJECTILE)
			return ProjectilesHitBoxes;
		else if(type == HitBoxType.PICKUP)
			return PickupHitBoxes;
		else
			return null;
	}
}
