/***************************************************************************
* A(lchemist) Game, a game about taking elements from the surrounding and throwing to a your enemies.
* Copyright (C) 2012  Sven Stamm
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
***************************************************************************/
package arakash.lpc.components;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.SpriteSheet;
import org.newdawn.slick.state.StateBasedGame;

import arakash.lpc.base.LPCAnimation;
import arakash.lpc.base.MyGame;
import arakash.lpc.base.ResourceManager;
import arakash.lpc.base.SpriteSheetPlus;
import arakash.lpc.entity.Entity;

/**
 * Assumes the spritesheets are like the LPC Defaults with 4 lines, with 
 * 1=up, 2=left, 3=down, 4=right
 * look for property on owner entity: resourceName
 * @author sven
 */
public class LPCSpriteSheetRenderComponent extends RenderComponent {
	//TODO this spritesheet is more like exclusive for walking animations?
	private SpriteSheet 	spriteSheet;
	private int			animTimePassed;
	private int 			TimeForOneAnim; 
	private int			oldX = 0;
	private int			oldY = 0;
	private boolean		standing = false;
	private boolean		hasStandingSprite = false;
	protected	String		curResourceName;
	protected boolean		norepeat =false;
	protected boolean		animEnd = false;
	
	public LPCSpriteSheetRenderComponent(String resourceName){
		this(ComponentList.LPCSPRITESHEET_RENDER_COMPONENT, resourceName);
	}
	
	public LPCSpriteSheetRenderComponent(int type, String resourceName){
		super(type);
		this.curResourceName = resourceName;
		LPCAnimation anim = ResourceManager.getAnimationFromResName(resourceName);
		this.setSpriteSheet(anim.getSpriteSheet());
		this.TimeForOneAnim = anim.getTimeForOneAnim();
		hasStandingSprite = anim.isHasStandingSprite();
		this.norepeat = anim.hasNorepeat();
		if(norepeat) System.out.println("got a ssrc with No Repeat!");
	}
	
	@Deprecated
	public LPCSpriteSheetRenderComponent(SpriteSheet ss, int OneAnimDurationMS, boolean firstColumnStandingSprite){
		this(ComponentList.LPCSPRITESHEET_RENDER_COMPONENT, ss, OneAnimDurationMS, firstColumnStandingSprite);
	}
	@Deprecated
	public LPCSpriteSheetRenderComponent(int type, SpriteSheet ss, int OneAnimDurationMS, boolean firstColumnStandingSprite){
		super(type);
		this.setSpriteSheet(ss);
		this.TimeForOneAnim = OneAnimDurationMS;
		hasStandingSprite = firstColumnStandingSprite;
	}
	
	@Override
	public void render(GameContainer gc, StateBasedGame sbgame, Graphics g) {
		int spriteToDraw = 0; 	//which sprite in the current line we want to draw
		if(standing && hasStandingSprite){
			spriteToDraw = 0;
		}
		else{
			if(TimeForOneAnim <= 0){
				spriteToDraw = (hasStandingSprite?1:0);
			}else{
				spriteToDraw = (int) (animTimePassed/TimeForOneAnim) + (hasStandingSprite?1:0);
			}
		}
		try{
		if(owner.getDIR() == Entity.DIR_UP)
			spriteSheet.getSprite(spriteToDraw, 0%spriteSheet.getVerticalCount()).draw(owner.getPosition().x - MyGame.getCameraPosX(), owner.getPosition().y - MyGame.getCameraPosY());
		if(owner.getDIR() == Entity.DIR_LEFT)
			spriteSheet.getSprite(spriteToDraw, 1%spriteSheet.getVerticalCount()).draw(owner.getPosition().x - MyGame.getCameraPosX(), owner.getPosition().y - MyGame.getCameraPosY());				
		if(owner.getDIR() == Entity.DIR_DOWN )
			spriteSheet.getSprite(spriteToDraw, 2%spriteSheet.getVerticalCount()).draw(owner.getPosition().x - MyGame.getCameraPosX(), owner.getPosition().y - MyGame.getCameraPosY());
		if(owner.getDIR() == Entity.DIR_RIGHT)
			spriteSheet.getSprite(spriteToDraw, 3%spriteSheet.getVerticalCount()).draw(owner.getPosition().x - MyGame.getCameraPosX(), owner.getPosition().y - MyGame.getCameraPosY());
		}
		catch(NullPointerException e){
			e.printStackTrace();
			System.out.println("NUllPointer Exception, while drawing. " + spriteSheet + ":" + owner.getProperty("resourceName"));
		}
	}

	@Override
	public void update(GameContainer gc, StateBasedGame sbgame, int delta) {
		if(oldX == (int) owner.getPosition().x && oldY == (int) owner.getPosition().y){
				standing = true;
		}else{
			oldX = (int) owner.getPosition().x;
			oldY = (int) owner.getPosition().y;
			standing = false;
		}
		
		if(owner.getProperty("resourceName")!= null && !curResourceName.equals(owner.getProperty("resourceName"))){
			
			LPCAnimation anim= ResourceManager.getAnimationFromResName(owner.getProperty("resourceName"));
			System.out.println("changing Resource! on " + owner.getID() + " from " + curResourceName + " to " + owner.getProperty("resourceName") + " at " + owner.getPosition());
			this.setSpriteSheet(anim.getSpriteSheet());
			this.TimeForOneAnim = anim.getTimeForOneAnim();
			hasStandingSprite = anim.isHasStandingSprite();
			norepeat = anim.hasNorepeat();
			animTimePassed=0;
			curResourceName = owner.getProperty("resourceName");
			animEnd=false;
		}

		if(!animEnd)
			animTimePassed += delta;
		if(spriteSheet == null){
			System.out.println("WARNING: trying to update on a NULL SpriteSheet! " + owner.getProperty("resourceName"));
			return;
		}
			
		//the first Sprite is the standing stance, so dont count that
		if(animTimePassed >= TimeForOneAnim * (spriteSheet.getHorizontalCount()-(hasStandingSprite?1:0))){
			if(norepeat){
				//set animTimePassed to the last image
				animTimePassed = TimeForOneAnim*(spriteSheet.getHorizontalCount()-1);
				animEnd = true;
			}
			else{
				animTimePassed = 0;
				animEnd = false;	//just for safety
			}
		}
	}

	@Override
	public void cleanup() {
	}

	public SpriteSheet getSpriteSheet() {
		return spriteSheet;
	}

	public void setSpriteSheet(SpriteSheet spriteSheet) {
		this.spriteSheet = spriteSheet;
	}
}
