/***************************************************************************
* A(lchemist) Game, a game about taking elements from the surrounding and throwing to a your enemies.
* Copyright (C) 2012  Sven Stamm
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
***************************************************************************/
package arakash.lpc.components;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.geom.Shape;
import org.newdawn.slick.geom.Vector2f;
import org.newdawn.slick.state.StateBasedGame;

import arakash.lpc.entity.Entity;

/**
 * Component who moves in the Direction the Entity faces.
 * check for Propertys on Entity:
 * - death
 * - speed
 * @author sven
 */
public class MovementComponent extends HitBox {
	float speed;
	
	public MovementComponent(int compType, Shape hitboxshape, HitBoxType type,
			int offsetX, int offsetY) {
		super(compType, hitboxshape, type, offsetX, offsetY);
	}

	@Override
	public void update(GameContainer gc, StateBasedGame sbgame, int delta) {
		if(owner.getProperty("death")!= null && owner.getProperty("death").equals("1"))
			return;		//owner is dead, so no Movement for you
		
		super.update(gc, sbgame, delta);
		this.speed = (float) Integer.parseInt(owner.getProperty("speed")) /100;

		if(owner.getDIR() == Entity.DIR_DOWN){
        	this.collisionShape.setY(this.collisionShape.getY() + speed * delta);

        	if((owner.getProperty("flying") == null || !Boolean.parseBoolean(owner.getProperty("flying"))) && 
        			(this.collisionShape.getY()+this.collisionShape.getHeight() >= map.getMapTotalHeight()-map.getTileHeight() || this.WorldCollideDown())){
        		this.collisionShape.setY(this.collisionShape.getY() - speed * delta);
        	}     
		}
		if(owner.getDIR() == Entity.DIR_LEFT){
    		this.collisionShape.setX(this.collisionShape.getX() - speed * delta);
    		//the outher Tiles are off-limit for you
        	if((owner.getProperty("flying") == null || !Boolean.parseBoolean(owner.getProperty("flying"))) && 
        			(this.collisionShape.getX() <= map.getTileWidth() || this.WorldCollideLeft()))
        		this.collisionShape.setX(this.collisionShape.getX() + speed * delta);
		}
		if(owner.getDIR() == Entity.DIR_UP){
        	this.collisionShape.setY(this.collisionShape.getY() - speed * delta);
        	if((owner.getProperty("flying") == null || !Boolean.parseBoolean(owner.getProperty("flying"))) && 
        			(this.collisionShape.getY() <= map.getTileHeight() || this.WorldCollideUp())){
        		this.collisionShape.setY(this.collisionShape.getY() + speed * delta);
        	}
		}
		if(owner.getDIR() == Entity.DIR_RIGHT){
       		this.collisionShape.setX(this.collisionShape.getX()+ speed * delta);
       		if((owner.getProperty("flying") == null || !Boolean.parseBoolean(owner.getProperty("flying"))) && 
       				(this.collisionShape.getX()+this.collisionShape.getWidth() >= map.getMapTotalWidth()-map.getTileWidth() || this.WorldCollideRight()))
       			this.collisionShape.setX(this.collisionShape.getX() - speed * delta);
		}
		
		owner.setPosition(new Vector2f(this.collisionShape.getX()-offsetX, this.collisionShape.getY()-offsetY));
	}
}
