/***************************************************************************
* A(lchemist) Game, a game about taking elements from the surrounding and throwing to a your enemies.
* Copyright (C) 2012  Sven Stamm
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
***************************************************************************/
package arakash.lpc.components;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.state.StateBasedGame;

import arakash.lpc.network.server.GameServer;
import arakash.lpc.network.server.PropertyChangeUpdate;
import arakash.lpc.network.shared.MessageUpdate;

/**
 * Server Component, <br>
 * checks if the Player has died and respawns him. Also sends a server Message saying so to all clients.
 * @author sven
 *
 */
public class PlayerDeathComponent extends Component {

	public PlayerDeathComponent(){
		super(ComponentList.PLAYER_DEATH_COMPONENT);
	}
	
	public PlayerDeathComponent(int type) {
		super(type);
	}

	@Override
	public void update(GameContainer gc, StateBasedGame sbgame, int delta) {
		if(owner.getProperty("damage") != null && owner.getProperty("health") != null){
			String newhealth = ""+ (Integer.parseInt(owner.getProperty("health")) - Integer.parseInt(owner.getProperty("damage")));
			owner.setOrAddProperty("health", newhealth);
			owner.setOrAddProperty("damage", "0");
		}
		
		// TODO make a superclass with smth. like "abstract onDeath"
		if(owner.getProperty("health") != null && Integer.parseInt(owner.getProperty("health")) < 0){
			owner.setOrAddProperty("health", owner.getProperty("maxhealth"));	//nicht wichtig für Client
			//send via PropertyChangeUpdate
			owner.setOrAddProperty("death", "1");					//auch wichtig für EINEN Client
			owner.setOrAddProperty("speed", "0");					//wichtig für EINEN Client, nicht fürn rest!
			owner.setOrAddProperty("resourceName", "mageDeath");	//wichtig für ALLE Clienten und Server (andere Clients sollen gleiche Anim sehen)
			owner.setOrAddProperty("score", "" + (Integer.parseInt(owner.getProperty("score"))/2));
			
			PropertyChangeUpdate pcu = new PropertyChangeUpdate();
			String[] propNames = new String[4];
			String[] propValues= new String[4];
			propNames[0] = "death"; propValues[0] = "1";
			propNames[1] = "speed"; propValues[1] = "0";
			propNames[2] = "resourceName"; propValues[2] = "mageDeath";
			propNames[3] = "score";	propValues[3] = ""+ (Integer.parseInt(owner.getProperty("score"))/2);
			pcu.setEntityID(owner.getID());
			pcu.setPropertyName(propNames); pcu.setPropertyValue(propValues);
			GameServer.addUpdate(pcu);
			
			MessageUpdate mu = new MessageUpdate();
			mu.setOwnerID(-1);
			mu.setMsg(owner.getProperty("name") + " was killed.");
			GameServer.addUpdate(mu);
		}
	}

	@Override
	public void cleanup() {
		// TODO Auto-generated method stub

	}
}
