/***************************************************************************
* A(lchemist) Game, a game about taking elements from the surrounding and throwing to a your enemies.
* Copyright (C) 2012  Sven Stamm
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
***************************************************************************/
package arakash.lpc.components;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.state.StateBasedGame;

import arakash.lpc.entity.Entity;
import arakash.lpc.network.server.GameServer;
import arakash.lpc.network.server.PropertyChangeUpdate;

/**
 * used to change a Property from a entity for a period of time.
 * can be a debuff (like ice) which wears out after a few seconds,
 * or a buff by a weapon, which wears off by unequipping<br>
 * <b>Server Component</b>
 * @author sven
 *
 */

public class PropertyChangeComponent extends Component {

	/**
	 * replaces the current Property with the given one
	 */
	public final static short PROP_REPLACE = 1;
	/**
	 * adds the given PropValue to the back of the Property
	 */
	public final static short PROP_CONCAT = 2;
	
	private int		TTLMS;
	private String		propName;
	private String		newPropValue;
	final	short		changeType;
	private int		Timer = 0;
	private String		oldValue;
	
	public PropertyChangeComponent(int TTLMS, String propName, String newPropValue, short changeType){
		this(ComponentList.PROPERTY_CHANGE_COMPONENT, TTLMS, propName, newPropValue, changeType);
	}
	
	public PropertyChangeComponent(int type, int TTLMS, String propName, String newPropValue,short changeType) {
		super(type);
		this.TTLMS = TTLMS;
		this.propName = propName;
		this.newPropValue = newPropValue;
		this.changeType = changeType;
	}

	@Override
	public void onAdd(Entity e){
		//TODO this doesn't work at all if 2 of these components target the same property!
		PropertyChangeUpdate pcu = new PropertyChangeUpdate();
		pcu.setEntityID(owner.getID());
		pcu.setPropertyName(new String[]{propName});
		
		oldValue = e.getProperty(propName);
		//set the new Property
		if(changeType == PROP_REPLACE){
			e.setOrAddProperty(propName, newPropValue);
			pcu.setPropertyValue(new String[]{""+newPropValue});
		}
		else if(changeType == PROP_CONCAT){
			String newVal = e.getProperty(propName) + "," + newPropValue;
			e.setOrAddProperty(propName, newVal);
			pcu.setPropertyValue(new String[]{newVal});
		}
		GameServer.addUpdate(pcu);
	}
	
	
	@Override
	public void update(GameContainer gc, StateBasedGame sbgame, int delta) {
		//tick down the Timer
		Timer+=delta;
		if(Timer >= TTLMS){
			PropertyChangeUpdate pcu = new PropertyChangeUpdate();
			pcu.setEntityID(owner.getID());
			pcu.setPropertyName(new String[]{propName});
			//restore the old Property. Replace the current with the old, if PROP_REPLACE was the type
			//and just remove yourself, if PROP_CONCAT was the changetype
			if(changeType == PROP_REPLACE){
				owner.setOrAddProperty(propName, oldValue);
				pcu.setPropertyValue(new String[]{oldValue});
			}
			if(changeType == PROP_CONCAT){
				owner.setOrAddProperty(propName, owner.getProperty(propName).replace(","+newPropValue, ""));
				System.out.println("removed added property, current line: " + owner.getProperty(propName));
				pcu.setPropertyValue(new String[]{owner.getProperty(propName).replace(","+newPropValue, "")});
			}
			owner.removeComponent(this);
			GameServer.addUpdate(pcu);
		}
	}

	@Override
	public void cleanup() {
		// TODO Auto-generated method stub

	}

	public int getTTLMS() {
		return TTLMS;
	}

	public void setTTLMS(int tTLMS) {
		TTLMS = tTLMS;
	}

	public String getPropName() {
		return propName;
	}

	public void setPropName(String propName) {
		this.propName = propName;
	}

	public String getNewPropValue() {
		return newPropValue;
	}

	public void setNewPropValue(String newPropValue) {
		this.newPropValue = newPropValue;
	}
}
