/***************************************************************************
* A(lchemist) Game, a game about taking elements from the surrounding and throwing to a your enemies.
* Copyright (C) 2012  Sven Stamm
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
***************************************************************************/
package arakash.lpc.entity;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Image;
import org.newdawn.slick.geom.Vector2f;
import org.newdawn.slick.state.StateBasedGame;

import arakash.lpc.base.PropertyListener;
import arakash.lpc.components.Component;
import arakash.lpc.components.ImageRenderComponent;
import arakash.lpc.components.RenderComponent;
import arakash.lpc.components.TextComponent;

public class Entity implements Serializable{
	public transient static final short DIR_UP = 0;
	public transient static final short DIR_LEFT=1;
	public transient static final short DIR_DOWN=2;
	public transient static final short DIR_RIGHT=3;
	
	protected int ID;
	protected int type;
	
	float positionX;
	float positionY;
	public int width;
	public int height;
	
	protected short DIR;

	transient static int numEntities = 0;		//used to generate EntityIds on the fly
	
	//TODO make this transient
	Hashtable<String,String>	Properties = new Hashtable<String,String>();	//nicht threadsafe, dafür HashTable benutzen
	
	transient TextComponent Text= null;
	
	transient ArrayList<RenderComponent> rendercomponents = null;
	
	transient ArrayList<Component> components = null;
	
	transient public ArrayList<Component>	componentsToDelete = new ArrayList<Component>();
	
	transient private ArrayList<PropertyListener> listeners = new ArrayList<PropertyListener>();
	
	/**
	 * adds automatical a ImageRenderComponent to this Entity.
	 * If you dont want this, use the other constructor and add the Component yourself
	 * @param img the visual representation of the Entity
	 */
	public Entity(Image img){
		this();
		this.addComponent(new ImageRenderComponent(img));
		this.width = img.getWidth();
		this.height= img.getHeight();
	}
	
	public Entity() {
		components = new ArrayList<Component>();
		rendercomponents=new ArrayList<RenderComponent>();
	}
	
	private Entity(int id){
		components = new ArrayList<Component>();
		
		this.ID = id;
		
		positionX = 0;
		positionY = 0;
	}
	
	public void addComponent(Component c){
		if(c instanceof RenderComponent){
			rendercomponents.add((RenderComponent) c);
		}
		
		c.setOwnerEntity(this);
		components.add(c);
		c.onAdd(this);
	}
	
	public void removeComponent(Component c){
		componentsToDelete.add(c);
	}
	
	public Component getComponent(int id){
		for(Component comp : components){
			if(comp.getID() == id)
				return comp;
		}
		
		return null;
	}
	
	public int getID(){ 
		return ID;
	}
		
	public Vector2f getPosition() {
		return new Vector2f(positionX,positionY);
	}

	public void setPosition(Vector2f position) {
		this.positionX = position.getX();
		this.positionY = position.getY();
	}

	public int getType(){
		return this.type;
	}
	
	public void setType(int type){
		this.type = type;
	}
	
	public void update(GameContainer gc, StateBasedGame sbgame, int delta){
		for(int i = 0; i < components.size(); i++){
			components.get(i).update(gc, sbgame, delta);
		}
		for(int i = 0; i< componentsToDelete.size(); i++){
			components.remove(componentsToDelete.get(i));
			componentsToDelete.get(i).setOwnerEntity(null);

			if(rendercomponents.contains(componentsToDelete.get(i))){
				rendercomponents.remove(componentsToDelete.get(i));
			}
			componentsToDelete.remove(componentsToDelete.get(i));
			i--;
		}
	}
	
	public void render(GameContainer gc, StateBasedGame sbgame, Graphics g){
			for(int index = 0; index < rendercomponents.size(); index++){
				rendercomponents.get(index).render(gc, sbgame, g);
			}
	}

	public short getDIR() {
		return DIR;
	}

	public void setDIR(short DIR) {
		this.DIR = DIR;
	}
	
	public void cleanup(){
		System.out.println("clenupping ent");
		for(int i = 0; i < components.size(); i++){
			components.get(i).cleanup();
		}
		components.clear();
		rendercomponents.clear();
	}
	
	public Entity copy(){
		Entity e = new Entity(this.getID());
		e.setPosition(this.getPosition());
		e.width = this.width;
		e.height= this.height;
		e.Properties= this.Properties;
		return e;
	}
	
	public boolean equals(Entity e){
		if( (int)this.getPosition().x == (int)e.getPosition().x &&
			(int)this.getPosition().y == (int)e.getPosition().y &&
			this.getID() == e.getID() &&
			this.width == e.width &&
			this.height== e.height){
			
			return true;
		}
		return false;
	}

	/**
	 * copys all values from the given Entity.
	 * @param e
	 */
	public void copyValues(Entity e){
		this.setPosition(e.getPosition());
		this.width = e.width;
		this.height= e.height;
		this.DIR = e.DIR;
	}
	
	public void copyProperties(Entity e){
		for(String key : e.Properties.keySet()){
			this.setOrAddProperty(key, e.getProperty(key));
		}
	}
	
	public void setOrAddProperty(String key, String value){
		Properties.put(key, value);
		for(PropertyListener pl : listeners){
			if(pl.getProperty().equals(key)){
				pl.onPropertyChange(value);
			}
		}
	}
	
	public String getProperty(String key){
		if(Properties == null)
			System.out.println("FATAL!! Properties sin null!!");
		return Properties.get(key);
	}
	
	public static Entity createNewEntity(){
		Entity e= new Entity();
		
		e.ID = numEntities;
		System.out.println("created new Entity");
		e.positionX = 0;
		e.positionY = 0;
		e.DIR= 0;
		numEntities++;
		return e;
	}
	
	public boolean isPropertyZeroOrNull(String propName){
		if(this.getProperty(propName) == null || this.getProperty(propName).equals("0"))
			return true;
		
		return false;
	}
	
	public void addPropertyListener(PropertyListener pl){
		listeners.add(pl);
		pl.setOwner(this);
	}
}
