/***************************************************************************
* A(lchemist) Game, a game about taking elements from the surrounding and throwing to a your enemies.
* Copyright (C) 2012  Sven Stamm
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
***************************************************************************/
package arakash.lpc.network.client;

import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.security.acl.Owner;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Set;


import org.newdawn.slick.AngelCodeFont;
import org.newdawn.slick.Color;
import org.newdawn.slick.Font;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.SpriteSheet;
import org.newdawn.slick.geom.Rectangle;
import org.newdawn.slick.gui.TextField;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.tiled.TiledMap;

import TWLSlick.BasicTWLGameState;
import TWLSlick.RootPane;
import arakash.lpc.Map.CameraControlComponent;
import arakash.lpc.Map.GameMap;
import arakash.lpc.base.EntityManagerInterface;
import arakash.lpc.base.MMManager;
import arakash.lpc.base.MyGame;
import arakash.lpc.base.PropertyListener;
import arakash.lpc.base.QuestManager;
import arakash.lpc.base.ResourceManager;
import arakash.lpc.components.ComponentList;
import arakash.lpc.components.HitBox;
import arakash.lpc.components.InterpolComponent;
import arakash.lpc.components.LPCSpriteSheetRenderComponent;
import arakash.lpc.components.MagicComponent;
import arakash.lpc.components.PingComponent;
import arakash.lpc.components.PlayerPingComponent;
import arakash.lpc.components.TextComponent;
import arakash.lpc.components.healthbarComponent;
import arakash.lpc.components.selfdestructComponent;
import arakash.lpc.components.walkingComponent;
import arakash.lpc.entity.Entity;
import arakash.lpc.entity.EntityManager;
import arakash.lpc.network.server.BigUpdate;
import arakash.lpc.network.server.EndGameUpdate;
import arakash.lpc.network.server.PropertyChangeUpdate;
import arakash.lpc.network.server.SoundUpdate;
import arakash.lpc.network.server.welcomeClient;
import arakash.lpc.network.shared.EntityUpdate;
import arakash.lpc.network.shared.MessageUpdate;
import arakash.lpc.network.shared.PlayerUpdate;
import arakash.lpc.network.client.helloServer;

import com.esotericsoftware.kryo.Kryo;
import com.esotericsoftware.kryonet.Client;
import com.esotericsoftware.kryonet.Connection;
import com.esotericsoftware.kryonet.Listener;
import com.esotericsoftware.kryonet.rmi.ObjectSpace;

import de.matthiasmann.twl.EditField;
import de.matthiasmann.twl.Label;
import de.matthiasmann.twl.ListBox;
import de.matthiasmann.twl.TextWidget;
import de.matthiasmann.twl.model.SimpleChangableListModel;

//TODO: error Msg. when unable to connect

public class ClientNetworkState extends BasicTWLGameState{

	public static final int ID = 20;
	
	static EntityManager entityManager = EntityManager.createNewManager();
	static MMManager	mmManager		= MMManager.createNewManager();
	
    static Entity Player;
    static TiledMap background;	
//    static ArrayList<Entity> otherPlayers = new ArrayList<Entity>();
//    static ArrayList<Entity> otherEntities= new ArrayList<Entity>();
    
    static ArrayList<Object> NetworkMessagesToSend = new ArrayList<Object>();
    
    public String DebugText = new String();
    public InetAddress serveraddress;
    
    public String MapToLoad = null;
	
	GameMap Map = null;
	
	int networkUpdateTimer = 0;
	int networkUpdateIntervall = 30;
	
	int reconnectIntervall = 1000;
	int reconnectTimer = 200;

	//ObjectSpace for remote Method Invocation
	ObjectSpace objSpace = new ObjectSpace();
	
	Font font;
	TextField lastWords;
	
	private StateBasedGame game = null;

	int changeState= -1;
	/***********
	 * GUI stuff
	 */
//    private Button btn;
    private 	Label		QuestLabel;
    private 	EditField DeathEF;	
    private 	TextWidget DeathText;
    private 	ListBox<String> scoreList;
    private 	SimpleChangableListModel<String> scoreModel;
    private 	ListBox<String> testBox;
    private 	SimpleChangableListModel<String> listmodel;
    public 		ArrayList<ListBox<String>>	guiBoxes; 
    private		ListBox<String> statWindow;
    
    @Override
    protected RootPane createRootPane() {
        RootPane rp = super.createRootPane();
        System.out.println("set theme... " + rp.getTheme());
        rp.setTheme("myTheme");
        guiBoxes=new ArrayList<ListBox<String>>();
//        btn = new Button("OK");
//        btn.setTheme("myButton");
        QuestLabel = new Label("this is mofo Quest!");
        QuestLabel.setTheme("TextField");
        rp.add(QuestLabel);
        
        scoreModel= new SimpleChangableListModel<String>();
        scoreList = new ListBox<String>(scoreModel);
        scoreList.setTheme("TextField");
        rp.add(scoreList);
        
        statWindow = new ListBox<String>(new SimpleChangableListModel<String>());
        statWindow.setTheme("TextField");
        ((SimpleChangableListModel<String>) statWindow.getModel()).addElement("Fire Level: " + "0");
        ((SimpleChangableListModel<String>) statWindow.getModel()).addElement("Water Level: " + "0");
        ((SimpleChangableListModel<String>) statWindow.getModel()).addElement("Earth Level: " + "0");
        ((SimpleChangableListModel<String>) statWindow.getModel()).addElement("kills : " + "0");
        rp.add(statWindow);
        
        DeathEF = new EditField();
        DeathEF.setTheme("TextField");
        DeathText = new TextWidget();
        DeathEF.add(DeathText);
        rp.add(DeathEF);

        listmodel = new SimpleChangableListModel<String>(); 
        testBox = new ListBox<String>(listmodel);
        testBox.setTheme("TextField");
        rp.add(testBox);

        return rp;
    }
    
    @Override
    protected void layoutRootPane() {
    	
    	scoreList.setPosition(10, 50);
    	scoreList.adjustSize();
    	
    	QuestLabel.setPosition(10, 150);
    	QuestLabel.adjustSize();
    	
        DeathEF.setSize(200, 50);
        DeathEF.setPosition(MyGame.getWIDTH()/2, 300);
        DeathEF.setVisible(false);
        
        testBox.setSize(400, 100);
        testBox.setPosition(10, MyGame.getHEIGHT()-testBox.getHeight()-10);
        
        statWindow.setPosition(800, 10);
        statWindow.adjustSize();
    }

	
	@Override
	public void enter(GameContainer arg0, StateBasedGame arg1)
			throws SlickException {
		super.enter(arg0, arg1);
		
		QuestManager.initTestQuestManager();
		// conn to server
		MyGame.setClient(new Client());
		MyGame.getClient().start();
		serveraddress = MyGame.address;
		try {
			MyGame.getClient().connect(5000, serveraddress.getHostAddress(), 54555, 54777);
		} catch (IOException e) {
			e.printStackTrace();
			//TODO: need feedback
			game.enterState(1);
		}
		Kryo kryo = MyGame.getClient().getKryo();
		ObjectSpace.registerClasses(kryo);
		kryo.register(EntityManagerInterface.class);
		kryo.register(helloServer.class);
		kryo.register(welcomeClient.class);
		kryo.register(PlayerUpdate.class);
		kryo.register(EntityUpdate.class);
		kryo.register(EntityUpdate[].class);
		kryo.register(Entity.class);
		kryo.register(Entity[].class);
		kryo.register(MagicAbsorbUpdate.class);
		kryo.register(MagicAttackUpdate.class);
		kryo.register(Hashtable.class);
		kryo.register(MessageUpdate.class);
		kryo.register(Object[].class);
		kryo.register(BigUpdate.class);
		kryo.register(PropertyChangeUpdate.class);
		kryo.register(String[].class);
		kryo.register(DeathMessageUpdate.class);
		kryo.register(SoundUpdate.class);
		kryo.register(EndGameUpdate.class);

//		System.out.println("Klasse mit ID 54: " + kryo.getRegisteredClass(54).getType());
		
		arg0.getInput().clearKeyPressedRecord();
		objSpace.register(42, EntityManager.curManager);
		// send a new hello server if not receiving a answer from the Server
		System.out.println("sending hello Server");
		//MyGame.getClient().sendTCP(new helloServer());
		MyGame.getClient().addListener(new Listener(){			
			   public void received (Connection connection, Object object) {
				   //I'm losing welcome clients here... even though i send them via TCP
				      if (object instanceof welcomeClient) {
				    	  if(Player!=null){
				    		  System.out.println("WARNING: ignoring welcome Clients, because i already have a Player");
				    		  return;
				    	  }
				  			objSpace.addConnection(connection);
				    	  welcomeClient wc = (welcomeClient) object;
				    	  System.out.println("got welcome Client, my Id is " + wc.getPlayerEntity().getID());
				    	  MapToLoad = wc.getMapPath();
				    	  System.out.println("maptoload:" + MapToLoad + " " + wc.getPlayerEntity().getID());
						  Player = wc.getPlayerEntity();
						  //TODO this still is not optimal
						  SpriteSheet PlayerImage = (SpriteSheet) ResourceManager.Resources.get("mageWalking");
						  Player.addComponent(new LPCSpriteSheetRenderComponent("mageWalking"));
						  Player.addComponent(new PlayerPingComponent());
						  Player.width = PlayerImage.getSprite(0, 0).getWidth();
						  Player.height= PlayerImage.getSprite(0, 0).getHeight();
						  Player.addComponent(new CameraControlComponent(400));
						  Player.addComponent(new walkingComponent(new Rectangle(Player.getPosition().x, Player.getPosition().y, Player.width-36, Player.height-43), 18,40));
						  Player.addComponent(new MagicComponent(ComponentList.MAGIC_COMPONENT, Input.KEY_SPACE));
						  
						  Player.addPropertyListener(new PropertyListener("firelvl") {							
							@Override
							public void onPropertyChange(String value) {
								((SimpleChangableListModel<String>)statWindow.getModel()).setElement(0, "Fire Level :" + value);
							}
						  });
						  Player.addPropertyListener(new PropertyListener("waterlvl") {							
							@Override
							public void onPropertyChange(String value) {
								((SimpleChangableListModel<String>)statWindow.getModel()).setElement(1, "Water Level :" + value);
							}
						  });
						  Player.addPropertyListener(new PropertyListener("earthlvl") {							
							@Override
							public void onPropertyChange(String value) {
								((SimpleChangableListModel<String>)statWindow.getModel()).setElement(2, "Earth Level :" + value);
							}
						  });
						  Player.addPropertyListener(new PropertyListener("kills") {
							@Override
							public void onPropertyChange(String value) {
								((SimpleChangableListModel<String>)statWindow.getModel()).setElement(3, "Kills :" + value);
							}
						  });
						  
						  MyGame.setCameraPosX(Player.getPosition().x-MyGame.getWIDTH()/2);
			    		  MyGame.setCameraPosY(Player.getPosition().y-MyGame.getHEIGHT()/2);
				      }
				      if(MapToLoad == null)
				    	  return;
				      //TODO fuse PlayUpdate and EntityUpdate
				      else if(object instanceof EntityUpdate){
				    	  processEntityUpdate((EntityUpdate) object, connection);
				      }
				      else if(object instanceof BigUpdate){
				    	  BigUpdate bu = (BigUpdate) object;
				    	  for(int i = 0; i < bu.getUpdates().length; i++){
				    		  this.received(connection, bu.getUpdates()[i]);
				    	  }
				      }
				      else if(object instanceof MessageUpdate){
				    	  MessageUpdate mu = (MessageUpdate) object;
				    	  if(mu.getOwnerID() == -1){
				    		  listmodel.addElement(mu.getMsg());
				    		  testBox.setSelected(listmodel.getNumEntries()-1);
				    	  }
				    	  else if(mu.getOwnerID() == Player.getID()){
				    		  Player.addComponent(new TextComponent(mu.getMsg(), 1, false, 0));
				    		  if(mu.isPing())
				    		  Player.addComponent(new PingComponent(mu.getDuration(), 0, false));
				    		  return;
				    	  }
				    	  else{
					    	  for(int i = 0; i < EntityManager.curManager.Entities.size(); i++){
					    		  if(EntityManager.curManager.Entities.get(i).getID() == mu.getOwnerID()){
					    			  EntityManager.curManager.Entities.get(i).addComponent(new TextComponent(mu.getMsg(), 2, false, 0));
					    			  if(mu.isPing())
					    			  EntityManager.curManager.Entities.get(i).addComponent(new PingComponent(mu.getDuration(), 0, false));
						    	  }
					    	  }
				    	  }
				      }
				      else if(object instanceof PropertyChangeUpdate){
				    	  PropertyChangeUpdate pcu = (PropertyChangeUpdate) object;
				    	  if(pcu.getEntityID() == Player.getID()){
				    		  for(int i = 0; i < pcu.getPropertyName().length; i++){
				    			  System.out.println("server said i should change " + pcu.getPropertyName()[i] + " to " + pcu.getPropertyValue()[i]);
				    			  Player.setOrAddProperty(pcu.getPropertyName()[i], pcu.getPropertyValue()[i]);
				    		  }
				    	  }
				    	  else{
				    		  for(int i = 0; i < EntityManager.curManager.getEntites().size(); i++){
				    			  if(EntityManager.curManager.getEntites().get(i).getID() == pcu.getEntityID()){
						    		  for(int u = 0; u < pcu.getPropertyName().length; u++){
						    			  Entity ent = EntityManager.curManager.getEntites().get(i);
						    			//TODO!! THIS IS REDUNDANT WITH ENTITY UPDATE! DANGER OF ADDING MULTIPLE INSTANCES OF THE SAME COMPONENT!!!!
						    			  if(pcu.getPropertyName()[u].equals("selfdestruct") && ent.getProperty("selfdestruct")==null)
						    				  EntityManager.curManager.getEntites().get(i).addComponent(new selfdestructComponent(Integer.parseInt(pcu.getPropertyValue()[u])));
						    			  else if(pcu.getPropertyName()[u].equals("soundResource"))
						    				  ResourceManager.Sounds.get(pcu.getPropertyValue()[u]).playAt(ent.getPosition().x-MyGame.getCameraPosX(), ent.getPosition().y-MyGame.getCameraPosY(), 1);
						    			  ent.setOrAddProperty(pcu.getPropertyName()[u], pcu.getPropertyValue()[u]);
						    		  }
				    			  }
				    		  }
				    	  }
				      }
				      else if( object instanceof SoundUpdate){
				    	SoundUpdate su = (SoundUpdate) object;
				    	//TODO need to differential between background Music (where only 1 should play) and effects
				    	if(su.isBackgroundMusic())
				    		ResourceManager.setBackgroundMusic(su.getSoundName());
				    	else if(su.isRepeat()){
				    		ResourceManager.loopSound(su.getSoundName());
				    	}
				    	else
				    		ResourceManager.playSound(su.getSoundName());
				      }
				      else if( object instanceof EndGameUpdate){
				    	  System.out.println("ENDING GAME");
				    	  EndGameUpdate egu = (EndGameUpdate) object;
				    	  MyGame.props.put("mostDeathsName", egu.getMostDeathsPlayerName());
				    	  MyGame.props.put("mostDeaths", "" + egu.getMostDeaths());
				    	  MyGame.props.put("bestScoreName", egu.getBestScorePlayerName());
				    	  MyGame.props.put("bestScore",""+ egu.getBestScore());
				    	  MyGame.props.put("mostKillsName", egu.getMostKillsPlayerName());
				    	  MyGame.props.put("mostKills",""+ egu.getMostKills());
				    	  MyGame.props.put("mostQuestsCompletedName", egu.getMostQuestsName());
				    	  MyGame.props.put("mostQuestsCompleted",""+ egu.getMostQuests());
				    	  
				    	  MyGame.props.put("myKills", Player.getProperty("kills"));
				    	  MyGame.props.put("myDeaths", Player.getProperty("playerDeaths"));
				    	  MyGame.props.put("myScore", Player.getProperty("score"));
				    	  MyGame.props.put("myQuestsCompleted", Player.getProperty("questsCompleted"));
				    	  
				    	  MyGame.props.put("alivePlayers", egu.getAlive());
				    	  MyGame.props.put("deadPlayers", egu.getDead());
				    	  changeState = 30;
				      }
				   }
				
			   @Override
				public void disconnected(Connection connection){
				   System.out.println("got disconnected");
					changeState = 1;
				}
		});
	}

	protected void processEntityUpdate(EntityUpdate object,
			Connection connection) {
		EntityUpdate eu = (EntityUpdate) object;
	  	Entity entity = eu.getEntity();
	  	
	  	boolean found = false;
	  	if(Player.getID() == entity.getID()){
	  		System.out.println("GOT ENTITY UPDATE WITH MY ID!!!!!");
	  		Player.copyValues(eu.getEntity());
	  		Player.copyProperties(eu.getEntity());
	  		MyGame.setCameraPosX(Player.getPosition().x -MyGame.getWIDTH()/2);
	  		MyGame.setCameraPosY(Player.getPosition().y - MyGame.getHEIGHT()/2);
	  		found = true;
	  	}
	  	for(int i = 0; i < EntityManager.curManager.Entities.size(); i++){
	  		Entity e = EntityManager.curManager.Entities.get(i);
	  		if(e.getID() == entity.getID()){
	  			e.copyValues(eu.getEntity());
	  			e.copyProperties(eu.getEntity());
				found = true;
		  	}
	  	}
	  	if(!found){
		  		  if(eu.getSelfdestructTimer() != 0){
		  			  entity.addComponent(new selfdestructComponent(eu.getSelfdestructTimer()));
		  		  }
		  		  String resourceName;
		  		  if(eu.getResourceName() != null){
		  			  resourceName = eu.getResourceName();
		  		  }
		  		  else
		  			  resourceName = eu.getEntity().getProperty("resourceName");
		  		  entity.addComponent(new LPCSpriteSheetRenderComponent(resourceName));
		  		  //if the entity has speed, we probalby have to interpol
		  		  if(entity.getProperty("speed") != null && !entity.getProperty("speed").equals("0")){
		  			entity.addComponent(new InterpolComponent(Integer.parseInt(entity.getProperty("speed"))));  
		  		  }
		  		  if(entity.getProperty("health") != null){
		  			  entity.addComponent(new healthbarComponent());
		  		  }
		  		  if(entity.getProperty("text") != null){
		  			  entity.addComponent(new TextComponent(entity.getProperty("text"), 10, true, 0));
		  		  }
		  		  if(entity.getProperty("soundResource") != null)
		  				 ResourceManager.Sounds.get(entity.getProperty("soundResource")).playAt(entity.getPosition().x - MyGame.getCameraPosX(), entity.getPosition().y - MyGame.getCameraPosY(), 0);
		  		  EntityManager.curManager.registerNewEntity(entity);
		  		  System.out.println("added new Entity to EntityManager " + entity.getID());
	  	}		
	}
	
	@Override
	public void render(GameContainer arg0, StateBasedGame arg1, Graphics arg2)
			throws SlickException {
		if(Map != null){
			int offsetX = (int) MyGame.getCameraPosX()%Map.getTileWidth();
			int offsetY = (int) (MyGame.getCameraPosY()%Map.getTileWidth());
			for(int i = 0; i <= Map.getLayerBelowPlayer(); i++){
				Map.render(-offsetX -Map.getTileWidth(), -offsetY -Map.getTileHeight(), 
						(int) (MyGame.getCameraPosX()/Map.getTileWidth())-1,(int) (MyGame.getCameraPosY()/Map.getTileHeight())-1, 
						MyGame.getWIDTH()/Map.getTileWidth()+3, MyGame.getHEIGHT()/Map.getTileHeight()+3, i,false);
			}
		}
		
		//TODO render otherEntities, enemys only if visible
		if(Player != null){
			Player.render(arg0, arg1, arg2);
			if(Player.getProperty("death")!= null && Player.getProperty("death").equals("1")){
				font.drawString(DeathEF.getX(), DeathEF.getY()-50, "any last Words?");
				font.drawString(MyGame.getWIDTH()/2, MyGame.getHEIGHT()/2," You are Dead!", Color.red);
			}
		}

		for(int i = 0; i < EntityManager.curManager.Entities.size(); i++){
			Entity e=EntityManager.curManager.Entities.get(i);
			e.render(arg0, arg1, arg2);
		}
		
		if(Map !=null){
			int offsetX = (int) MyGame.getCameraPosX()%Map.getTileWidth();
			int offsetY = (int) (MyGame.getCameraPosY()%Map.getTileWidth());
			for(int i = Map.getLayerBelowPlayer()+1; i < Map.getLayerCount(); i++){
			Map.render(-offsetX -Map.getTileWidth(), -offsetY -Map.getTileHeight(), 
					(int) (MyGame.getCameraPosX()/Map.getTileWidth())-1,(int) (MyGame.getCameraPosY()/Map.getTileHeight())-1, 
					MyGame.getWIDTH()/Map.getTileWidth()+3, MyGame.getHEIGHT()/Map.getTileHeight()+3, i,false);
			}
		}
	}

	@Override
	public void update(GameContainer arg0, StateBasedGame arg1, int delta)
			throws SlickException {
		if(Map == null && MapToLoad != null){
			initNewMap();
		}
		if(Player == null){
			if(reconnectTimer > reconnectIntervall){
				helloServer hs = new helloServer();
				hs.setPlayerName(MyGame.PlayerName);
				MyGame.getClient().sendTCP(hs);
				reconnectTimer = 0;
			}
			else
				reconnectTimer += delta;
			return;
		}
		if(Player != null){
			Player.update(arg0, arg1, delta);
			
//			ef.setText(Player.getProperty("firelvl") + Player.getProperty("waterlvl") + Player.getProperty("earthlvl"));
//			ef.setText("jadada" + '\n' + "lolol");
			if(Player.getProperty("death")!= null && Player.getProperty("death").equals("1") && arg0.getInput().isKeyPressed(Input.KEY_ENTER)){
					//send the death message to the server
					//TODO make you can send the Update only once
					DeathMessageUpdate dmu = new DeathMessageUpdate();
					dmu.setText(DeathEF.getText());
					MyGame.getClient().sendTCP(dmu);
					DeathEF.setText("");
					DeathEF.setVisible(false);
					System.out.println("hiding Death EditField");
					//we assume thats whats coming next
					Player.setOrAddProperty("death", "0");
			}else if(Player.getProperty("death") != null && Player.getProperty("death").equals("1"))
				DeathEF.setVisible(true); 
			
			/**
			 * SCORE TEST!!
			 */
			if(Player.getProperty("currentQuest")==null || Player.getProperty("currentQuest").equals(""))
				QuestLabel.setVisible(false);
			else{
				QuestLabel.setVisible(true); 
				QuestLabel.setText(QuestManager.getShortText(Integer.parseInt(Player.getProperty("currentQuest"))));
			}
		}
			
		for(int i = 0; i < EntityManager.curManager.Entities.size(); i++){
			EntityManager.curManager.Entities.get(i).update(arg0, arg1, delta);
		}
		EntityManager.curManager.finishEntityDeletion();
		if(networkUpdateTimer >= networkUpdateIntervall){
				PlayerUpdate pU = new PlayerUpdate();
				pU.setPlayerEntity(Player);
				MyGame.getClient().sendUDP(pU);
			networkUpdateTimer -= networkUpdateIntervall;
			
			while(NetworkMessagesToSend.size() > 0){
				MyGame.getClient().sendTCP(NetworkMessagesToSend.get(0));
				NetworkMessagesToSend.remove(0);
			}			
			scoreModel.clear();
			scoreModel.addElement(Player.getProperty("name") + ": " + Player.getProperty("score"));
			//TODO: rework this
			for(int i = 0; i < EntityManager.curManager.Entities.size(); i++){
				Entity e = EntityManager.curManager.Entities.get(i);
				if(e.getProperty("score") != null)
					scoreModel.addElement(e.getProperty("name") + ": " + e.getProperty("score"));
			}
		}
		
		networkUpdateTimer +=delta;
		arg0.getInput().clearKeyPressedRecord();
		if(changeState!= -1){
			game.enterState(changeState);
			changeState = -1;
		}
		if(Player != null && !MyGame.getClient().isConnected()){
			game.enterState(1);
		}
	}

	private void initNewMap() throws SlickException {
		// load new Map
		System.out.println("mtl: " + this.MapToLoad);
		Map = new GameMap(this.MapToLoad);
		// in Movement/HitBox set the Collision Layer, then check with the Map if there are Tiles in that Layer
		HitBox.map = Map;
		HitBox.collisionLayerId=Map.getCollisionLayerId();
	}

	public void showDeathGUI(GameContainer gc) throws SlickException{
		
	}
	
	@Override
	public int getID() {
		return ID;
	}

	@Override
	public void init(GameContainer arg0, StateBasedGame arg1)
			throws SlickException {
		font = new AngelCodeFont("data/SlickFont.fnt", "data/SlickFont_00.tga");
		
		game = arg1;
	}
	
	@Override
	public void leave(GameContainer arg0, StateBasedGame arg1)
			throws SlickException {
		if(Player != null) Player.cleanup();
		Player = null;
	    MapToLoad = null;
		Map = null;
		for(int i = 0; i< EntityManager.curManager.Entities.size(); i++){
			EntityManager.curManager.Entities.get(i).cleanup();
		}
		EntityManager.curManager.Entities.clear();
		this.rootPane = null;
/*		Set<String> keys = ResourceManager.Resources.keySet();
		for(String key : keys){
			ResourceManager.Resources.get(key).destroy();
			//TODO ensure its a spritesheet?
			if(ResourceManager.Animations.containsKey(key))
			(ResourceManager.Resources.get(key)).destroy();
		}*/
	}
}
