/***************************************************************************
* A(lchemist) Game, a game about taking elements from the surrounding and throwing to a your enemies.
* Copyright (C) 2012  Sven Stamm
* 
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
***************************************************************************/

package arakash.lpc.network.server;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
import java.util.Properties;
import java.util.Random;

import org.newdawn.slick.Game;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.geom.Rectangle;
import org.newdawn.slick.geom.Shape;
import org.newdawn.slick.geom.Vector2f;

import arakash.lpc.Map.GameMap;
import arakash.lpc.base.EntityManagerInterface;
import arakash.lpc.base.LPCAnimation;
import arakash.lpc.base.MMManager;
import arakash.lpc.base.QuestManager;
import arakash.lpc.base.ResourceManager;
import arakash.lpc.base.SpriteSheetPlus;
import arakash.lpc.components.HitBox;
import arakash.lpc.components.PlayerDeathComponent;
import arakash.lpc.components.scrollPickupComponent;
import arakash.lpc.components.HitBox.HitBoxType;
import arakash.lpc.components.selfdestructComponent;
import arakash.lpc.entity.Entity;
import arakash.lpc.entity.EntityList;
import arakash.lpc.entity.EntityManager;
import arakash.lpc.network.client.DeathMessageUpdate;
import arakash.lpc.network.client.MagicAbsorbUpdate;
import arakash.lpc.network.client.MagicAttackUpdate;
import arakash.lpc.network.client.helloServer;
import arakash.lpc.network.shared.EntityUpdate;
import arakash.lpc.network.shared.MessageUpdate;
import arakash.lpc.network.shared.PlayerUpdate;

import com.esotericsoftware.kryo.Kryo;
import com.esotericsoftware.kryonet.Connection;
import com.esotericsoftware.kryonet.Listener;
import com.esotericsoftware.kryonet.Server;
import com.esotericsoftware.kryonet.rmi.ObjectSpace;
import com.esotericsoftware.kryonet.rmi.RemoteObject;

public class GameServer implements Game{

	public GameServer() {
		
	}

	static EntityManager entityManager = EntityManager.createNewManager();
	static MMManager	mmManager = MMManager.createNewManager();
	static int maxNumPlayers = 1;
	public static ArrayList<PlayerClient> clients = new ArrayList<PlayerClient>();
	private static ArrayList<Object> UpdatesToAll = new ArrayList<Object>();
	
	public static Random randGenerator = new Random();
	
    int curNumPlayers = 0;
    boolean completeUpdate = false;
    
    //int responseId;
    public Server server = null;
    
    static String MapPath 	= null;
    static GameMap Map 		= null;		//TODO server can't dynamically change maps
    
    int		sendUpdateInterval = 30;
    int		updateTimer			=0;
    
	public static ObjectSpace objSpace = new ObjectSpace();
   
	public static boolean armageddon = false;
	int		armageddonTTL= 100000;
	int		armageddonTimer=0;
	
    @Override
    public void init(GameContainer gc) throws SlickException{
		Properties prop = new Properties();
		try {
			prop.load(new FileInputStream("ServerConfig.properties"));
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		int TCPPort = Integer.parseInt(prop.getProperty("serverportTCP", "54555"));
		int UDPPort = Integer.parseInt(prop.getProperty("serverportUDP", "54777"));
		
		server = new Server();
		server.start();
		try {
			server.bind(TCPPort, UDPPort);
		} catch (IOException e) {
			e.printStackTrace();
		}
		maxNumPlayers=Integer.parseInt(prop.getProperty("maxnumplayers", "4"));
		
		MapPath = prop.getProperty("map", "level/testlevel1.tmx");
		
		//responseId = 0;
		Kryo kryo = server.getKryo();

		ObjectSpace.registerClasses(kryo);
		kryo.register(EntityManagerInterface.class);
		kryo.register(helloServer.class);
		kryo.register(welcomeClient.class);
		kryo.register(PlayerUpdate.class);
		kryo.register(EntityUpdate.class);
		kryo.register(EntityUpdate[].class);
		kryo.register(Entity.class);
		kryo.register(Entity[].class);
		kryo.register(MagicAbsorbUpdate.class);
		kryo.register(MagicAttackUpdate.class);
		kryo.register(Hashtable.class);
		kryo.register(MessageUpdate.class);
		kryo.register(Object[].class);
		kryo.register(BigUpdate.class);
		kryo.register(PropertyChangeUpdate.class);
		kryo.register(String[].class);
		kryo.register(DeathMessageUpdate.class);
		kryo.register(SoundUpdate.class);
		kryo.register(EndGameUpdate.class);
		
		
		
		Map = new GameMap(MapPath);
		HitBox.map = Map;
		HitBox.collisionLayerId=Map.getCollisionLayerId();
		QuestManager.initTestQuestManager();
		QuestManager.generateNPCs();
		QuestManager.generateExkalibur();
		
		ResourceManager.init();
		   
		server.addListener(new Listener() {
			@Override	
			   public void connected(Connection connection){
					System.out.println("ich bin JETZT connected!");
			   }
			   public void received (Connection connection, Object object) {
				   if(object instanceof helloServer){
					   processHelloServer((helloServer) object, connection);
				   }
				   if(object instanceof PlayerUpdate){
					   PlayerUpdate pu = (PlayerUpdate) object;
					   EntityUpdate eu = new EntityUpdate();
					   eu.setEntity(pu.getPlayerEntity());
					   //TODO: i dont need the special field resourceName anymore
					   eu.setResourceName(pu.getPlayerEntity().getProperty("resourceName"));
					   boolean found= false;
					   for(PlayerClient pc : clients){
						   if(pc.getClientEntity().getID() == pu.getPlayerEntity().getID()){
							   pc.getClientEntity().setPosition(pu.getPlayerEntity().getPosition());
							   pc.getClientEntity().setDIR(pu.getPlayerEntity().getDIR());
							   eu.setEntity(pc.getClientEntity());
							   found = true;
						   }
					   }
					   if(!found)
						   clients.add(new PlayerClient(pu.getPlayerEntity(), connection, "unexpected connect"));
					   else{
						   for(PlayerClient pc : clients){
							   if(pc.getClientEntity().getID() != pu.getPlayerEntity().getID()){
								   	server.sendToUDP(pc.getClientConnection().getID(), eu);
							   }
						   }
					   }
				   }
				   if(object instanceof MagicAbsorbUpdate){
					   processMagicAbsorb((MagicAbsorbUpdate) object, connection);
				   }
				   if(object instanceof MagicAttackUpdate){
					   processMagicAttack((MagicAttackUpdate)object, connection);
				   }
				   //TODO better add all of this to a Updates to send package, so that all updates are sent in 1 bug package
				   if(object instanceof MessageUpdate){
					   addUpdate((MessageUpdate) object);
				   }
				   if(object instanceof DeathMessageUpdate){
					   DeathMessageUpdate dmu = (DeathMessageUpdate) object;
					   System.out.println("got an DeathMessageUpdate!! " + dmu.getText());
					   //search User
					   Entity playerEnt = null;
					   for(int i = 0; i < clients.size(); i++){
						   if(clients.get(i).getClientConnection().getID() == connection.getID()){
							   playerEnt = clients.get(i).getClientEntity();
						   }
					   }
					   //if we don't know the Player, or the update is corrupt, ignore it
					   if(playerEnt == null) return;
					   
					   //create Gravestone
						Entity Gravestone = Entity.createNewEntity();
						Integer playerLvl = playerEnt.getProperty("level") != null ? Integer.parseInt(playerEnt.getProperty("level")): 0;
						if(playerLvl <3)
						Gravestone.setOrAddProperty("resourceName", "gravestonesSmall");
						else if(playerLvl < 9)
						Gravestone.setOrAddProperty("resourceName", "gravestonesMedium");
						else
						Gravestone.setOrAddProperty("resourceName", "gravestonesBig");
		
						if(playerEnt.isPropertyZeroOrNull("level"))
							Gravestone.setDIR((short) 0);
						else if(playerEnt.getProperty("level").equals("1") || Integer.parseInt(playerEnt.getProperty("level")) >5)
							Gravestone.setDIR((short)1);
						else
							Gravestone.setDIR((short)2);
						//TODO set Player Name here!
						Gravestone.setOrAddProperty("text", "here lies " + playerEnt.getProperty("name") +": " + '\n' + dmu.getText());
						Gravestone.setPosition(playerEnt.getPosition());
						//TODO how do we manage the Text?
						EntityUpdate eu = new EntityUpdate();
						eu.setEntity(Gravestone);
						GameServer.addUpdate(eu);
						EntityManager.curManager.Entities.add(Gravestone);
						
						int oldLevel = playerEnt.getProperty("level")!=null ? Integer.parseInt(playerEnt.getProperty("level")) : 0;
						if(oldLevel < 6){
							
						}
						else if (oldLevel < 8){
							if(randGenerator.nextBoolean()){
								Entity enemy = MMManager.getInstance().generateMeAEnemy("blackArmor");
								enemy.setPosition(Gravestone.getPosition());
								enemy.setOrAddProperty("text", playerEnt.getProperty("name"));
								EntityManager.curManager.Entities.add(enemy);
								MessageUpdate mu = new MessageUpdate();
								mu.setMsg(playerEnt.getProperty("name") + " came back as a hellish creature");
								mu.setOwnerID(-1);
								addUpdate(mu);
							}
						}
						else{
							if(randGenerator.nextBoolean()){
								Entity enemy = MMManager.getInstance().generateMeAEnemy("maskman");
								enemy.setPosition(Gravestone.getPosition());
								enemy.setOrAddProperty("text", playerEnt.getProperty("name"));
								EntityManager.curManager.Entities.add(enemy);
								MessageUpdate mu = new MessageUpdate();
								mu.setMsg(playerEnt.getProperty("name") + " came back as a horrible demon");
								mu.setOwnerID(-1);
								addUpdate(mu);	
							}
						}
						
					   //set his Propertys again
						playerEnt.setOrAddProperty("speed", "20");
						playerEnt.setOrAddProperty("death", "0");
						playerEnt.setOrAddProperty("resourceName", "mageWalking");
						playerEnt.setOrAddProperty("firelvl", "0");
						playerEnt.setOrAddProperty("waterlvl", "0");
						playerEnt.setOrAddProperty("earthlvl", "0");
						playerEnt.setOrAddProperty("level", "0");
						playerEnt.setOrAddProperty("playerDeaths",""+(Integer.parseInt(playerEnt.getProperty("playerDeaths"))+1));	
						int deaths = Integer.parseInt(playerEnt.getProperty("playerDeaths"));
						String affix;
						if(deaths+1 == 2)
							affix = "nd";
						else if(deaths+1 == 3)
							affix = "rd";
						else
							affix = "th";
						playerEnt.setOrAddProperty("name", playerEnt.getProperty("playerName") + " the " + (Integer.parseInt(playerEnt.getProperty("playerDeaths"))+1) + affix);
					   //set his new Position
						playerEnt.setPosition(GameMap.CurrentMap.getRandPointInSpawnPoint(randGenerator.nextInt(GameMap.CurrentMap.getNumSpawnPoints())));
						eu = new EntityUpdate();
						eu.setEntity(playerEnt);
					   //send him his Update
						server.sendToTCP(connection.getID(), eu);
				   }
			   }
			   
			//deregister when player disconnects
			   public void disconnected(Connection connection){
				   int PlayerID = -1;
					for(int i=0; i < clients.size(); i++){
						if(clients.get(i).getClientConnection().getID() == connection.getID()){
							PlayerID=clients.get(i).getClientEntity().getID();
							System.out.println("deregistering connection with id " + connection.getID());
							clients.get(i).getClientEntity().cleanup();
							clients.remove(i);
							break;
						}
					}
					for(PlayerClient pc : clients){
						EntityManager.curManager.deregisterEntity(PlayerID);
						EntityManagerInterface em = ObjectSpace.getRemoteObject(pc.getClientConnection(), 42, EntityManagerInterface.class);
						((RemoteObject) em).setNonBlocking(true, true);
						em.deregisterEntity(PlayerID);
						System.out.println("done");
						return;
					}
			   }
		});
	}
	

	private void processHelloServer(helloServer object, Connection connection) {
		// check if there is room
		boolean found = false;
		this.completeUpdate = true;
		Entity Player = null;
		for(PlayerClient pc : clients){
			if(pc.getClientConnection().getID() == connection.getID()){
				Player = pc.getClientEntity();
				found = true;
				break;
			}
		}
		if(!found){
			Player = Entity.createNewEntity();
			Player.setOrAddProperty("resourceName", "mageWalking");
			System.out.println("new Player with ID:" + Player.getID());
			Random Generator = new Random();
			Player.setPosition(Map.getRandPointInSpawnPoint(Generator.nextInt(Map.getNumSpawnPoints())));
			Player.setType(EntityList.PLAYER_TYPE);
			Shape PlayerHitBoxShape = new Rectangle(Player.getPosition().x, Player.getPosition().y, 48,32);		//TODO also pretty hardcoded
			Player.addComponent(new HitBox(PlayerHitBoxShape, HitBoxType.PLAYER, 8, 36));						//TODO extremly hardcoded with the above player hitboxshape
			Player.addComponent(new PlayerDeathComponent());
			Player.setOrAddProperty("health", "1000");			//player health in config or smth?
			Player.setOrAddProperty("damage", "0");
			Player.setOrAddProperty("maxhealth", "1000");
			Player.setOrAddProperty("speed", "20");
			Player.setOrAddProperty("playerName", object.getPlayerName());
			Player.setOrAddProperty("playerDeaths", "0");
			Player.setOrAddProperty("name", object.getPlayerName() + " the 1st");
			Player.setOrAddProperty("score", "0");
			clients.add(new PlayerClient(Player, connection, object.getPlayerName()));
		}
		welcomeClient wc = new welcomeClient();
		// send him the map
		wc.setMapPath(MapPath);
		// send him the Player
		wc.setPlayerEntity(Player);
		System.out.println("sending welcome Client");
		connection.sendUDP(wc);
		
		MessageUpdate mu = new MessageUpdate();
		mu.setOwnerID(-1);
		mu.setMsg("A new Hero has entered! (" + Player.getProperty("name") + ")");
		GameServer.addUpdate(mu);
	}
	
	public void processMagicAbsorb(MagicAbsorbUpdate mau, Connection connection){
		 // Server: EntityUpdate (an alle) [als type FireAbsorb,waterAbsorb,earthabsorb?] eig. kann der Client das selber checken
		//TODO server doesn't check how much elements a client has
		Entity e = Entity.createNewEntity();
		e.setPosition(new Vector2f(mau.getTargetX(), mau.getTargetY()));
		LPCAnimation anim = ResourceManager.Animations.get("magicAbsorb");
		e.addComponent(new selfdestructComponent(anim.getTimeForOneAnim()*anim.getSpriteSheet().getHorizontalCount()));		//TODO hardcoded ...
		EntityUpdate eu = new EntityUpdate();
		
		int TargetTileX= (int) (mau.getTargetX()/GameMap.CurrentMap.getTileWidth());
		int TargetTileY= (int) (mau.getTargetY()/GameMap.CurrentMap.getTileHeight());
		if(GameMap.CurrentMap.elementalTilePos.get(new Vector2f(TargetTileX, TargetTileY)).equals("water")){
			e.setDIR(Entity.DIR_UP);
			e.setOrAddProperty("soundResource", "waterBubble");
		}
		else if(GameMap.CurrentMap.elementalTilePos.get(new Vector2f(TargetTileX, TargetTileY)).equals("earth")){
			e.setDIR(Entity.DIR_LEFT);
			e.setOrAddProperty("soundResource", "earthBubble");
		}
		else if(GameMap.CurrentMap.elementalTilePos.get(new Vector2f(TargetTileX, TargetTileY)).equals("fire")){
			e.setDIR(Entity.DIR_DOWN);
			e.setOrAddProperty("soundResource", "fireBubble");
		}
		else
			return;
		
		EntityManager.curManager.registerNewEntity(e);
		eu.setResourceName("magicAbsorb");
		eu.setEntity(e);
		eu.setSelfdestructTimer(600);
		addUpdate(eu);
	}

	/**
	 * distributes the Magic Attack to all Clients, as well as checking for Hits
	 * @param mau
	 * @param connection
	 */
	protected void processMagicAttack(MagicAttackUpdate mau,
			Connection connection) {
		Entity e = MMManager.getInstance().generateMeAMagic(mau.getMagicName(), mau.getPosX(), mau.getPosY());
		for(int i = 0; i < clients.size(); i++){
			if(connection.getID() == clients.get(i).getClientConnection().getID()){
				e.setOrAddProperty("ownerID", ""+clients.get(i).getClientEntity().getID());
			}
		}
		EntityManager.curManager.registerNewEntity(e);
	}
	
	public static void main(String[] args) throws SlickException{
		ServerGameContainer app =
				new ServerGameContainer( new GameServer() );
		app.setMinimumLogicUpdateInterval(10);
//		app.setMaximumLogicUpdateInterval(10);
		app.start();
	}
	
	@Override
	public void update(GameContainer arg0, int delta) throws SlickException{
		
		//update Entities and shit
		//update everything
		if(clients.size() == 0) return;
			Entity e1 = null;
		//System.out.println("updating!! " + delta);
			//with this Method we will lose updates everytime an Entity is removed from the List
			//but in theory we do update so fast that this one lost update shouldn't make a difference
			//to reduce networkusage, only send in intervalls and send all updates in one big package
			if(updateTimer >= sendUpdateInterval){
				for(int i = 0; i < EntityManager.curManager.Entities.size(); i++){
					Entity e = EntityManager.curManager.Entities.get(i);
					e1=e.copy();
					e.update(null, null, (int) delta);
					if(completeUpdate || (e != null && !e1.equals(e))){
						EntityUpdate eu = new EntityUpdate();
						eu.setEntity(e);
						eu.setResourceName(e.getProperty("resourceName"));
						addUpdate(eu);
					}
				}
				if(UpdatesToAll.size() > 0){
					//send all Updates in a big UpdatePackage, upack this on the client (by calling 'received' multiple times)
					BigUpdate bu = new BigUpdate();
					//one "big" package is about 10 updates in size
					//TODO checkForComodificationException here :///
					synchronized (UpdatesToAll) {
						for(int i = 0; i*10 < UpdatesToAll.size(); i++){
							List<Object> subList = UpdatesToAll.subList(i*10, (i+1)*10 > UpdatesToAll.size()?UpdatesToAll.size():(i+1)*10);
							bu.setUpdates(subList.toArray(new Object[subList.size()]));
							for(int u = 0; u< clients.size(); u++){
								server.sendToUDP(clients.get(u).getClientConnection().getID(), bu);
							}
						}
					}
//					if(UpdatesToAll.size() > 10)
//						System.out.println("WARN: nearing cirtical package size! " + UpdatesToAll.size() + " and delta: " + delta);
		//			bu.setUpdates(UpdatesToAll.toArray(new Object[UpdatesToAll.size()]));
		//			for(int i = 0; i< clients.size(); i++){
		//				server.sendToUDP(clients.get(i).getClientConnection().getID(), bu);
		//			}
					UpdatesToAll.clear();
				}
				updateTimer = 0;
				//approx. every 30 Updates, make a complete update
				
				if(randGenerator.nextInt(60) == 1){
					completeUpdate = true;
					//System.out.println("complete Update!");
				}else
					completeUpdate = false;
			}
			else{
				for(int i = 0; i < EntityManager.curManager.Entities.size(); i++){
					Entity e = EntityManager.curManager.Entities.get(i);
					e.update(arg0, null, delta);
				}
			}
			updateTimer+=delta;
			EntityManager.curManager.finishEntityDeletion();
			for(PlayerClient pc : clients){
				pc.getClientEntity().update(arg0, null, delta);
			}
			if(HitBox.EnemyHitBoxes.size() < 20){
				generateEnemy("Slime");
				generateEnemy("Skeleton");
				generateEnemy("Bat");
//				if(randGenerator.nextInt(2) == 1)	//slow down the amount of scrolls in the world 
					generateScroll(randGenerator.nextInt(3));
				System.out.println("adding new Entities to populate the world! currently: " + EntityManager.curManager.Entities.size());
			}
			
			if(armageddon){
				for(int i = 0; i < clients.size(); i++){
					testSafePoint(i);
				}
				if(armageddonTimer >= armageddonTTL){
					finishArmageddon();
				}
				else if(randGenerator.nextInt(500) == 1){
					MessageUpdate mu = new MessageUpdate();
					mu.setOwnerID(-1);
					mu.setMsg("You have " + (armageddonTTL - armageddonTimer)/1000 +" seconds left to reach the SafePoint");
					addUpdate(mu);
				}
				armageddonTimer+=delta;
			}
	}

	@Override
	public void render(GameContainer arg0, Graphics arg1) throws SlickException {
		return;
	}

	@Override
	public boolean closeRequested() {
		System.exit(1);
		return false;
	}

	@Override
	public String getTitle() {
		return "Server";
	}
	
	public void generateEnemy(String name){
		Random generator = new Random();
		Entity enemy = MMManager.getInstance().generateMeAEnemy(name);
		enemy.setPosition(GameMap.CurrentMap.getRandPointInEnemySpawn(generator.nextInt(GameMap.CurrentMap.getNumEnemySpawns())));
		EntityManager.curManager.registerNewEntity(enemy);
	}
	
	/**
	 * generates a elemental scroll
	 * @param scroll 0=water, 1=earth, 2=fire, (4=gold), see scrollPickupComponent
	 */
	public static void generateScroll(int scroll){
		Entity e = Entity.createNewEntity();
		SpriteSheetPlus ssp = (SpriteSheetPlus) ResourceManager.Resources.get("elementalScroll");
		Shape scrollShape = new Rectangle(0, 0, ssp.getSpriteWidth(), ssp.getSpriteHeight());
		//TODO: well, technically this would fall into the Pickup class, but i dont see it beneficial here, probably i can just cut it all together
		e.addComponent(new scrollPickupComponent(scrollShape, 0, 0, HitBoxType.PLAYER, scroll));
		e.setDIR((short) scroll);
		e.setOrAddProperty("resourceName", "elementalScroll");
		e.setPosition(GameMap.CurrentMap.getRandPointInEnemySpawn(randGenerator.nextInt(GameMap.CurrentMap.getNumEnemySpawns())));
		EntityManager.curManager.registerNewEntity(e);
		//send it to da playas! 
		EntityUpdate eu = new EntityUpdate();
		eu.setEntity(e);
		eu.setResourceName("elementalScroll");
		addUpdate(eu);
	}
	
	private void testSafePoint(int playerindex){
		if(clients.get(playerindex).getClientEntity().getProperty("safe") != null && clients.get(playerindex).getClientEntity().getProperty("safe").equals("1")){
			return;
		}
		if (GameMap.CurrentMap.isPointInObject("SafePoint", 0,(int) clients.get(playerindex).getClientEntity().getPosition().x,(int) clients.get(playerindex).getClientEntity().getPosition().y)){
			MessageUpdate mu = new MessageUpdate();
			mu.setOwnerID(-1);
			mu.setMsg(clients.get(playerindex).getClientEntity().getProperty("name") + " has reached the SafePoint!");
			addUpdate(mu);
			clients.get(playerindex).getClientEntity().setOrAddProperty("safe", "1");
		}
	}
	
	private void finishArmageddon(){
		for(int i = 0; i < clients.size(); i++){
			if (GameMap.CurrentMap.isPointInObject("SafePoint", 0,(int) clients.get(i).getClientEntity().getPosition().x,(int) clients.get(i).getClientEntity().getPosition().y)){
				MessageUpdate mu = new MessageUpdate();
				mu.setOwnerID(-1);
				mu.setMsg(clients.get(i).getClientEntity().getProperty("name") + " survied the Armageddon!");
				addUpdate(mu);				
			}
			else{
				MessageUpdate mu = new MessageUpdate();
				mu.setOwnerID(-1);
				mu.setMsg(clients.get(i).getClientEntity().getProperty("name") + " hasn't reached the SafePoint and got annihilated");
				addUpdate(mu);
			}
		}
		generateStats();
	}
	//TODO: The server has to restart after the endgame...
	private void generateStats(){
		int mostDeaths= -1;
		int bestScore = -1;
		int mostKills = -1;
		int mostQuestsCompleted = -1;
		String Safe = "";
		String dead = "";
		//int highestLevel = 0;
		EndGameUpdate egu = new EndGameUpdate();
		for(int i = 0; i < clients.size(); i++){
			Entity clientEnt = clients.get(i).getClientEntity();
			if(clientEnt.getProperty("playerDeaths") != null && mostDeaths == -1)
				mostDeaths = i;
			else if(clientEnt.getProperty("playerDeaths") != null && 
					Integer.parseInt(clientEnt.getProperty("playerDeaths"))>Integer.parseInt(clients.get(mostDeaths).getClientEntity().getProperty("playerDeaths"))){
				mostDeaths = i;
			}
			if(clientEnt.getProperty("score") != null && bestScore == -1)
				bestScore = i;
			else if(clientEnt.getProperty("score") != null &&
					Integer.parseInt(clientEnt.getProperty("score"))>Integer.parseInt(clients.get(bestScore).getClientEntity().getProperty("score"))){
				bestScore = i;
			}
			if(clientEnt.getProperty("kills") != null && mostKills == -1)
				mostKills = i;
			else if(clientEnt.getProperty("kills") != null &&
					Integer.parseInt(clientEnt.getProperty("kills"))>Integer.parseInt(clients.get(mostKills).getClientEntity().getProperty("kills"))){
				mostKills = i;
			}
			if(clientEnt.getProperty("questsCompleted")!=null && mostQuestsCompleted == -1)
				mostQuestsCompleted = i;
			else if(clientEnt.getProperty("questsCompleted")!=null &&
					Integer.parseInt(clientEnt.getProperty("questsCompleted"))>Integer.parseInt(clients.get(mostQuestsCompleted).getClientEntity().getProperty("questsCompleted"))){
				mostQuestsCompleted=i;
			}
			if (GameMap.CurrentMap.isPointInObject("SafePoint", 0,(int) clients.get(i).getClientEntity().getPosition().x,(int) clients.get(i).getClientEntity().getPosition().y)){
				Safe += ", " + clientEnt.getProperty("name");
			}
			else
				dead += ", " + clientEnt.getProperty("name");

		}
		
		egu.setMostDeaths(Integer.parseInt(clients.get(mostDeaths).getClientEntity().getProperty("playerDeaths")));
		egu.setMostDeathsPlayerName(clients.get(mostDeaths).getClientEntity().getProperty("name"));
		egu.setBestScore(Integer.parseInt(clients.get(bestScore).getClientEntity().getProperty("score")));
		egu.setBestScorePlayerName(clients.get(bestScore).getClientEntity().getProperty("name"));
		egu.setMostKills(Integer.parseInt(clients.get(mostKills).getClientEntity().getProperty("kills")));
		egu.setMostKillsPlayerName(clients.get(mostKills).getClientEntity().getProperty("name"));
		if(mostQuestsCompleted != -1){
			egu.setMostQuests(Integer.parseInt(clients.get(mostQuestsCompleted).getClientEntity().getProperty("questsCompleted")));
			egu.setMostQuestsName(clients.get(mostQuestsCompleted).getClientEntity().getProperty("name"));
		}
		else{
			egu.setMostQuests(0);
			egu.setMostQuestsName("nobody");
		}
		egu.setAlive(Safe.equals("") ? "nobody": Safe);
		egu.setDead(dead.equals("") ? "nobody" : dead);
		
		server.sendToAllTCP(egu);
		this.restart();
	}
	
	public static void addUpdate(Object update){
		synchronized (UpdatesToAll) {
			UpdatesToAll.add(update);
		}
	}
	public void restart(){
//		server.close();
		armageddon = false;
		armageddonTTL= 100000;
		armageddonTimer=0;
//		objSpace = new ObjectSpace();
		entityManager = EntityManager.createNewManager();
		UpdatesToAll.clear();
		QuestManager.initTestQuestManager();
		QuestManager.generateNPCs();
		QuestManager.generateExkalibur();
	}
}
